################################################################################
#!/bin/bash                                                                    #
#                                                                              #
# This script automatically installs and configures a high-security router     #
# with the following software: Apache HTTPS, Apparmor, Asterisk VoIP,          #
# Barnyard, ClamAV, DDclient, GnuPG, ISC Dynamic Bind DNS, ISC Dynamic DHCP,   #
# LTSP, Munin, MySQL, Nagios, NFS, OpenSSL, OpenVPN, phpMyAdmin, Postfix SMTP, #
# Privoxy, ProFTPD, PyGoogleVoice, Samba CIFS, Shorewall, Snorby, Snort,       #
# Squid, SquidClamAV, SquidGuard, SquidGuardMgr, and Webmin.                   #
#                                                                              #
################################################################################
#                                                                              #
# Copyfree 2016 T.L. Bean                                                      #
#                                                                              #
################################################################################
#                                                                              #
# This script supports Ubuntu Server with two Ethernet adapters.               #
#                                                                              #
# If there is any error condition, view install.log for further details.       #
#                                                                              #
# This script uses Bash functions that do not work with the Dash shell.        #
# To switch from Dash to Bash, use the following command:                      #
# sudo dpkg-reconfigure dash                                                   #
#                                                                              #
################################################################################
#                                                                              #
# Manually adjust variables:                                                   #
#                                                                              #
SERVER_ADMIN="Administrator"                                                   #
EMAIL_ADDRESS="admin@example.com"                                              #
ORGANIZATION="Example"                                                         #
# Use only two-digit country code.                                             #
COUNTRY="US"                                                                   #
# Spell out the state in-full.                                                 #
STATE="Oregon"                                                                 #
HOSTNAME="or-rt1-ub"                                                           #
LAN_INTERFACE="enp0s1"                                                         #
LAN_IP_ADDRESS="192.168.0.1"                                                   #
LAN_NETMASK="255.255.255.0"                                                    #
WAN_INTERFACE="enp0s2"                                                         #
# Use www.speedof.me to determine the up/down bandwidth (kilobits per second). #
WAN_INTERFACE_BANDWIDTH_UP="4000"                                              #
WAN_INTERFACE_BANDWIDTH_DOWN="40000"                                           #
# Set fastest Ubuntu mirror.                                                   #
UBUNTU_MIRROR="http://archive.ubuntu.com"                                      #
#                                                                              #
################################################################################

# Check for Internet connection.
ping -q -w 1 -c 1 `ip r | grep default | cut -d' ' -f 3` > /dev/null
if [ $? -ne 0 ]; then
  clear
  echo ""
  echo "Please check your Internet connection."
  echo ""
  exit 1
fi

# Set administrator password.
clear
echo ""
read -sp "Enter your administrator password: " ADMIN_PASSWORD1
clear
echo ""
read -sp "Re-enter administrator password to verify: " ADMIN_PASSWORD2
while [ "$ADMIN_PASSWORD1" != "$ADMIN_PASSWORD2" ]; do
  clear
  echo ""
  echo "Your passwords do not match, please try again."
  echo ""
  read -sp "Enter your administrator password: " ADMIN_PASSWORD1
  clear
  echo ""
  read -sp "Re-enter administrator password to verify: " ADMIN_PASSWORD2
done
ADMIN_PASSWORD=$ADMIN_PASSWORD1

( # Redirect STDOUT and STDERROR logged to terminal and install.log.

echo "Script start time: $(date +%c)"
echo ""

cat > /etc/apt/preferences << EOF.apt.pinning
# LTSP's Dnsmasq interferes with Bind DNS.
Package: dnsmasq
Pin: version *
Pin-Priority: -100
EOF.apt.pinning

# Apt function to fetch binary software packages.
apt_function() {
  APT="apt-get --yes --allow-unauthenticated \
    -o Dpkg::Options::=--force-confdef,overwrite"
  APT_ARRAY=($*)
  $APT ${APT_ARRAY[*]}
  while [ $? -ne 0 ]; do
    clear
    echo ""
    echo "Unable to connect to software repository, please wait."
    echo ""
    sleep 30
    apt clean all
    rm -f /var/cache/apt/{archives,partial}/lock
    dpkg --force-confdef,overwrite --configure -a
    $APT ${APT_ARRAY[*]}
  done
  return 0
}

# Wget function to fetch and extract .tar.gz archives.
WGET="wget --progress=bar:force --continue --tries=0 \
  --no-dns-cache --no-check-certificate --retry-connrefused"
wget_tar_function() {
  $WGET $1 -O /usr/local/src/$2.tar.gz
  tar xzf /usr/local/src/$2.tar.gz
  while [ $? -ne 0 ]; do
    clear
    echo ""
    echo "Unable to connect to software repository, please wait."
    echo ""
    sleep 60
    rm -rf /usr/local/src/$2*
    $WGET $1 -O /usr/local/src/$2.tar.gz
    tar xzf /usr/local/src/$2.tar.gz
  done
  return 0
}

# Wget function to fetch .deb packages.
wget_deb_function() {
  $WGET $1 -O /usr/local/src/$2.deb
  dpkg -i /usr/local/src/$2.deb
  while [ $? -ne 0 ]; do
    clear
    echo ""
    echo "Unable to connect to software repository, please wait."
    echo "" 
    sleep 30
    rm -f /usr/local/src/$2.deb
    $WGET $1 -O /usr/local/src/$2.deb
    dpkg -i /usr/local/src/$2.deb
  done
  rm -f /usr/local/src/$2.deb
  return 0
}

# SVN function to fetch source software packages.
svn_function() {
  SVN_ARRAY=($*)
  svn --trust-server-cert --non-interactive --force export ${SVN_ARRAY[*]}
  while [ $? -ne 0 ]; do
    clear
    echo ""
    echo "Unable to connect to software repository, please wait."
    echo "" 
    sleep 60
    svn --trust-server-cert --non-interactive --force export ${SVN_ARRAY[*]}
  done
  return 0
}

# GIT function to fetch source software packages.
git_function() {
  GIT_ARRAY=($*)
  git clone --verbose ${GIT_ARRAY[*]}
  while [ $? -ne 0 ]; do
    clear
    echo ""
    echo "Unable to connect to software repository, please wait."
    echo "" 
    sleep 60
    git clone --verbose ${GIT_ARRAY[*]}
  done
  return 0
}

# Function to convert subnet mask to CIDR format.
mask2cidr() {
  NBITS=0
  IFS=.
  for IP_ADDRESS_4TH_OCTET in $1; do
    case $IP_ADDRESS_4TH_OCTET in
      255) let NBITS+=8;;
      254) let NBITS+=7;;
      252) let NBITS+=6;;
      248) let NBITS+=5;;
      240) let NBITS+=4;;
      224) let NBITS+=3;;
      192) let NBITS+=2;;
      128) let NBITS+=1;;
      0);;
      *) exit 1
    esac
  done
  echo "$NBITS"
  return 0
}

# Function to get network address.
get_network_address() {
  SAVE_IFS=$IFS
  IFS=.
  typeset -a IP_ADDRESS_ARRAY=($1)
  typeset -a NETMASK_ARRAY=($2)
  IFS=$SAVE_IFS
  echo $((${IP_ADDRESS_ARRAY[0]} & ${NETMASK_ARRAY[0]})).$((${IP_ADDRESS_ARRAY[1]} & \
    ${NETMASK_ARRAY[1]})).$((${IP_ADDRESS_ARRAY[2]} & \
    ${NETMASK_ARRAY[2]})).$((${IP_ADDRESS_ARRAY[3]} & ${NETMASK_ARRAY[3]}))
  return 0
}

# Function to get broadcast address.
get_broadcast_address() {
  SAVE_IFS=$IFS
  IFS=.
  typeset -a IP_ADDRESS_ARRAY=($1)
  typeset -a NETMASK_ARRAY=($2)
  IFS=$SAVE_IFS
  echo $((${IP_ADDRESS_ARRAY[0]} | (255 ^ ${NETMASK_ARRAY[0]}))).$((${IP_ADDRESS_ARRAY[1]} | \
    (255 ^ ${NETMASK_ARRAY[1]}))).$((${IP_ADDRESS_ARRAY[2]} | \
    (255 ^ ${NETMASK_ARRAY[2]}))).$((${IP_ADDRESS_ARRAY[3]} | \
    (255 ^ ${NETMASK_ARRAY[3]})))
  return 0
}

# Set variables.
OS_RELEASE=$(lsb_release -c -s)
HARDWARE_PLATFORM=$(uname -i)
ARCHITECTURE=$(dpkg --print-architecture)
CIDR=$(mask2cidr $LAN_NETMASK)
LAN_NETWORK_ADDRESS=$(get_network_address $LAN_IP_ADDRESS $LAN_NETMASK)
LAN_BROADCAST=$(get_broadcast_address $LAN_IP_ADDRESS $LAN_NETMASK)
IFS=. read -a LAN_IP_ADDRESS_OCTETS <<< "$LAN_IP_ADDRESS"
IFS=. read -a LAN_NETWORK_ADDRESS_OCTETS <<< "$LAN_NETWORK_ADDRESS"
LAN_IP_ADDRESS_4TH_OCTET=${LAN_IP_ADDRESS_OCTET[3]}
LAN_REVERSE_ZONE=${LAN_IP_ADDRESS_OCTETS[2]}.${LAN_IP_ADDRESS_OCTETS[1]}.\
${LAN_IP_ADDRESS_OCTETS[0]}.in-addr.arpa
DHCP_HOST_MIN=${LAN_NETWORK_ADDRESS_OCTETS[0]}.${LAN_NETWORK_ADDRESS_OCTETS[1]}.\
${LAN_NETWORK_ADDRESS_OCTETS[2]}.1
DHCP_HOST_MAX=${LAN_IP_ADDRESS_OCTETS[0]}.${LAN_IP_ADDRESS_OCTETS[1]}.\
${LAN_IP_ADDRESS_OCTETS[2]}.$(($(echo $LAN_BROADCAST | cut -d\. -f4) - 1))
LAN_DOMAIN=$(echo "$ORGANIZATION" | sed 's| ||g' \
| tr '[:upper:]' '[:lower:]').corp
WORKGROUP=$(echo "$ORGANIZATION" | sed 's| ||g' \
| tr '[:lower:]' '[:upper:]')
HOSTNAME=$(echo $HOSTNAME | tr '[:upper:]' '[:lower:]')
DYNAMICDNS_HOST=$HOSTNAME.dnsdynamic.com
FQDN=$HOSTNAME.$LAN_DOMAIN

# Add admin user account (used for Webmin).
useradd -Mg admin admin
echo -ne "$ADMIN_PASSWORD\n$ADMIN_PASSWORD\n" | passwd admin

# Set hostname.
hostnamectl set-hostname $HOSTNAME

# Configure hosts.
if [ ! -f /etc/hosts.orig ]; then mv /etc/hosts /etc/hosts.orig; fi
cat << EOF.hosts | column -t > /etc/hosts
127.0.0.1	localhost.localdomain	localhost
$LAN_IP_ADDRESS	$FQDN	$(echo $HOSTNAME | tr '[:upper:]' '[:lower:]')
EOF.hosts

# Configure Sysctl.
if [ ! -f /etc/sysctl.conf.orig ]; then 
  mv /etc/sysctl.conf /etc/sysctl.conf.orig
fi
cat > /etc/sysctl.conf << EOF.sysctl.conf
# Set domain name.
kernel.domainname = $LAN_DOMAIN
# Enable IP forwarding.
net.ipv4.ip_forward = 1
# Do not accept ICMP redirects (prevent MITM attacks).
net.ipv4.conf.all.accept_redirects = 0
EOF.sysctl.conf
sysctl -p /etc/sysctl.conf

# Prevent interactive dialog.
export DEBIAN_FRONTEND=noninteractive

# Update Apt sources.list.
if [ ! -f /etc/apt/sources.list.orig ]; then 
  cp /etc/apt/sources.list /etc/apt/sources.list.orig
fi
sed -i "s|http://*.*.archive.ubuntu.com|$UBUNTU_MIRROR|g" \
  /etc/apt/sources.list

# Add LTSP repository.
add-apt-repository -ny ppa:ltsp

# Add Webmin Apt repository.
echo "deb http://download.webmin.com/download/repository sarge contrib" \
  > /etc/apt/sources.list.d/webmin.list
$WGET -qO- http://www.webmin.com/jcameron-key.asc | sudo tee /etc/apt/trusted.gpg.d/jcameron-key.asc

# Resynchronize the package index files from their sources.
apt_function update

# Define software package variables.
APACHE="apache2 apache2-doc apache2-utils libapache2-mod-authnz-external \
  libapache2-mod-authz-unixgroup libapache2-mod-fcgid libapache2-mod-passenger \
  libapache2-mod-php8.1"
APPARMOR="apparmor apparmor-notify apparmor-profiles apparmor-utils \
  dh-apparmor"
ASTERISK="asterisk asterisk-dahdi asterisk-doc dahdi-dkms"
BARNYARD_DEPENDS="autoconf build-essential libdaq-dev libdumbnet-dev \
  libmysqlclient-dev libmysqlclient21 libpcap-dev libprelude-dev libtool"
BIND="bind9 bind9-doc bind9utils"
CHRONY="chrony"
CLAMAV="clamav clamav-daemon clamav-docs clamav-freshclam"
CUPS="cups"
DDCLIENT="ddclient"
DHCPD="isc-dhcp-server"
GNUPG="gnupg2 gnupg-agent gpgv rng-tools"
LTSP="ltsp ltsp-binaries tftpd-hpa"
MUNIN="ethtool libcgi-fast-perl libnet-cidr-perl libnet-ssleay-perl munin \
  munin-node munin-plugins-extra smartmontools"
MYSQL="automysqlbackup mysql-client mysql-server mysqltuner phpmyadmin"
NAGIOS="nagios4 nagios4-cgi monitoring-plugins"
NFS="nfs-kernel-server nfs4-acl-tools"
OPENSSL="openssl"
OPENVPN="openvpn"
PHP="php8.1 php8.1-dev"
POSTFIX="postfix postfix-doc procmail"
PROFTPD="proftpd-basic proftpd-doc proftpd-mod-clamav"
PROXY="c-icap privoxy squid squidguard squidguard-doc"
PYGOOGLEVOICE_DEPENDS="python-setuptools"
SAMBA="cifs-utils samba"
SHOREWALL="shorewall shorewall-doc shorewall-init"
SNORBY_DEPENDS="imagemagick libmysqlclient-dev libxml2-dev libxslt1-dev \
  ruby-dev ruby-full rubygems-integration wkhtmltopdf"
SNORT="oinkmaster snort snort-doc"
SOURCE_CODE_DEPENDS="debhelper devscripts dpatch git gcc lintian make \
  subversion"
SQUIDCLAMAV_DEPENDS="libicapapi-dev libtimedate-perl"
SYSTEM_DEPENDS="apt-utils dkms openssh-server vim-scripts"
WEBMIN="cups mdadm quota quotatool sarg stunnel4 usermin webalizer webmin wodim"

# Software packages verification.
SOFTWARE_PACKAGES="$APACHE $APPARMOR $ASTERISK $BARNYARD_DEPENDS $BIND \
  $CHRONY $CLAMAV $CUPS $DDCLIENT $DHCPD $GNUPG $LTSP $MUNIN $MYSQL $NAGIOS \
  $NFS $OPENSSL $OPENVPN $PHP $POSTFIX $PROFTPD $PROXY $PYGOOGLEVOICE_DEPENDS \
  $SAMBA $SHOREWALL $SNORBY_DEPENDS $SNORT $SOURCE_CODE_DEPENDS \
  $SQUIDCLAMAV_DEPENDS $SYSTEM_DEPENDS $WEBMIN"

# Software package verification.
apt-get -s install $SOFTWARE_PACKAGES > /dev/null
if [ $? -ne 0 ]; then
  clear
  echo ""
  echo "Software packages verification problem detected."
  echo "View software.log for further details."
  echo ""
  echo "Analyze the packages in software.log. For example, if the log contains"
  echo "a package named php7.2, this means that php7.2 is not found on your" 
  echo "Ubuntu mirror. Packages may change over time. Use the 'apt-cache search'"
  echo "command to search for just the package name, without the version number."
  echo "For example, the 'apt-cache search php' command will show if there is"
  echo "an updated php package available. For your information, php7.2 is" 
  echo "available on Ubuntu 18.04 mirrors, and php7.4 is available on Ubuntu"
  echo "20.04 mirrors."
  echo ""
  echo "If the log contains the name of a package that does not show up when"
  echo "you search for it with 'apt-cache search', this means the package is"
  echo "most likely deprecated and needs to be removed from this script."
  echo ""
  apt-get -s install $SOFTWARE_PACKAGES 2> software.log
  exit 1
fi

### Automatic software packages. ###
# svn.digium.com/svn/asterisk-gui/trunk
ASTERISK_GUI="http://svn.digium.com/svn/asterisk-gui/trunk"
# github.com/firnsy/barnyard2
BARNYARD="https://github.com/firnsy/barnyard2.git"
# shallalist.de
SHALLA_BLACKLISTS="http://www.shallalist.de/Downloads/shallalist.tar.gz"
# snorby.org/#download
SNORBY="http://github.com/Snorby/snorby.git"
# sourceforge.net/projects/squidclamav
SQUIDCLAMAV="http://sourceforge.net/projects/squidclamav/files/latest/download"
# sourceforge.net/projects/squidguardmgr
SQUIDGUARDMGR="http://sourceforge.net/projects/squidguardmgr/files/latest\
/download"

clear
echo ""
echo "Installing prerequisite software packages."
echo ""

# Install software prerequisites.
apt_function install $SYSTEM_DEPENDS $SOURCE_CODE_DEPENDS

################################################################################
#                                                                              #
# The following routine installs and configures Chrony, which provides time    #
# synchronization.                                                             #
#                                                                              #
# chrony.tuxfamily.org                                                         #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing Chrony."
echo ""

# Install Chrony.
apt_function install $CHRONY

# Configure chrony.conf.
if [ ! -f /etc/chrony/chrony.conf.orig ]; then 
  cp /etc/chrony/chrony.conf /etc/chrony/chrony.conf.orig
fi
grep -q "add $LAN_NETWORK_ADDRESS/$CIDR" /etc/chrony/chrony.conf || \
echo "add $LAN_NETWORK_ADDRESS/$CIDR" >> /etc/chrony/chrony.conf

# Reload configuration.
systemctl restart chrony

################################################################################
#                                                                              #
# The following routine installs AppArmor, a Linux Security Module             #
# implementation of name-based mandatory access controls. AppArmor confines    #
# individual programs to a set of listed files and POSIX 1003.1e draft         #
# capabilities.                                                                #
#                                                                              #
# wiki.apparmor.net                                                            #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing AppArmor."
echo ""

# Install AppArmor.
apt_function install $APPARMOR

################################################################################
#                                                                              #
# The following routine installs and configures Shorewall, a high-level tool   #
# for configuring Netfilter. You describe your firewall/gateway requirements   #
# using entries in a set of configuration files. Shorewall reads those         #
# configuration files and with the help of the iptables utility, Shorewall     #
# configures Netfilter to match your requirements.                             #
#                                                                              #
# shorewall.org                                                                #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing Shorewall."
echo ""

# Install Shorewall.
apt_function install $SHOREWALL

# Configure shorewall.conf.
sed -i "s|LOG_MARTIANS=Yes|LOG_MARTIANS=No|
  s|STARTUP_ENABLED=No|STARTUP_ENABLED=Yes|" /etc/shorewall/shorewall.conf

# Create rules.
cat > /etc/shorewall/rules << EOF.rules
?SECTION NEW
#ACTION           SOURCE  DEST
Asterisk(ACCEPT)  lan    \$FW
CUPS(ACCEPT)      lan    \$FW
DHCPfwd(ACCEPT)   lan    \$FW
DNS(ACCEPT)       lan    \$FW
FTP(ACCEPT)       lan    \$FW
HTTP(ACCEPT)      lan    \$FW
HTTPS(ACCEPT)     lan    \$FW
Munin(ACCEPT)     lan    \$FW
MySQL(ACCEPT)     lan    \$FW
Nagios(ACCEPT)    lan    \$FW
NFS(ACCEPT)       lan    \$FW
NTP(ACCEPT)       lan    \$FW
OpenVPN(ACCEPT)   lan    \$FW
Ping(ACCEPT)      lan    \$FW
SMB(ACCEPT)       lan    \$FW
SMBBI(ACCEPT)     lan    \$FW
Squid(ACCEPT)     lan    \$FW
SSH(ACCEPT)       lan    \$FW
Syslog(ACCEPT)    lan    \$FW
TFTP(ACCEPT)      lan    \$FW
Webmin(ACCEPT)    lan    \$FW
REDIRECT          lan     3128  tcp  www - !$LAN_IP_ADDRESS

Asterisk(ACCEPT)  vpn    \$FW
CUPS(ACCEPT)      lan    \$FW
DHCPfwd(ACCEPT)   vpn    \$FW
DNS(ACCEPT)       vpn    \$FW
FTP(ACCEPT)       vpn    \$FW
HTTP(ACCEPT)      vpn    \$FW
HTTPS(ACCEPT)     vpn    \$FW
Munin(ACCEPT)     vpn    \$FW
MySQL(ACCEPT)     vpn    \$FW
Nagios(ACCEPT)    vpn    \$FW
NFS(ACCEPT)       vpn    \$FW
NTP(ACCEPT)       vpn    \$FW
OpenVPN(ACCEPT)   vpn    \$FW
Ping(ACCEPT)      vpn    \$FW
SMB(ACCEPT)       vpn    \$FW
SMBBI(ACCEPT)     vpn    \$FW
Squid(ACCEPT)     vpn    \$FW
SSH(ACCEPT)       vpn    \$FW
Syslog(ACCEPT)    vpn    \$FW
TFTP(ACCEPT)      vpn    \$FW
Webmin(ACCEPT)    vpn    \$FW
REDIRECT          vpn     3128  tcp  www - !$LAN_IP_ADDRESS

Ping(DROP)        wan    \$FW
OpenVPN(ACCEPT)   wan    \$FW
FTP(ACCEPT)       wan    \$FW
SSHKnock          wan    \$FW  tcp  22,1700,1701,1701

# PORT MAP
# 8 = ICMP (Internet Control Message Protocol)
# 21 = FTP (File Transfer Protocol is used by ProFTPD)
# 22 = SSH (Secure Shell)
# 25 = SMTP (Simple Mail Transport Protocol is used by Postfix)
# 53 = DNS (Domain Name Service is used by Bind)
# 67, 68 = DHCP (Dynamic Host Configuration Protocol)
# 69 = TFTP (Trivial File Transfer Protocol is used by LTSP)
# 80 = HTTP (Hypertext Transfer Protocol is used by Apache)
# 111 = RPCbind is used by NFS.
# 113 = IDENT (Identification Protocol is blocked to enhance security)
# 123 = NTP (Network Time Protocol)
# 443 = HTTPS (HTTP, secure)
# 445 = Microsoft-DS (Common Internet Mail Access Protocol is used by Samba)
# 514 = Syslog
# 631 = CUPS
# 1194 = OpenVPN
# 2000 = NBD Image Export for LTSP
# 2049 = NFS (Network File System)
# 3128 = Squid
# 3306 = MySQL
# 4000,4001 = RPC.statd is used by NFS.
# 4002 = RPC.mountd is used by NFS.
# 4569 = IAX/2 (Inter-Asterisk eXchange v2 is used by Asterisk VoIP)
# 4949 = Munin
# 5038, 8089 = Asterisk-GUI
# 5060, 5061 = SIP (Session Initiation Protocol is used by Asterisk VoIP)
# 5666 = Nagios
# 9571 = Ldminfod (login and locale settings for LTSP)
# 9572 = Nbdswapd (NBD swap for LTSP)
# 10000 = Webmin
# 10809 = NBD-server (Network Block Device)
# 20000 = Usermin
# 30000:40000 = RTP (Real Time Protocol is used by Asterisk VoIP)
EOF.rules

### Define macros ###

# Asterisk macro.
cat > /etc/shorewall/macro.Asterisk << EOF.asterisk.macro
?FORMAT 2
#ACTION SOURCE DEST PROTO DPORT
PARAM   -      -    tcp   4569
PARAM   -      -    tcp   5038
PARAM   -      -    tcp   5060
PARAM   -      -    udp   5060
PARAM   -      -    tcp   5061
PARAM   -      -    udp   5061
PARAM   -      -    tcp   8089
PARAM   -      -    tcp   30000:40000
PARAM   -      -    udp   30000:40000
EOF.asterisk.macro

# CUPS macro.
cat > /etc/shorewall/macro.CUPS << EOF.cups.macro
?FORMAT 2
#ACTION  SOURCE  DEST  PROTO  DPORT
PARAM    -       -     tcp    631
EOF.cups.macro

# LTSP macro.
cat > /etc/shorewall/macro.LTSP << EOF.ltsp.macro
?FORMAT 2
#ACTION SOURCE DEST PROTO DPORT
PARAM   -      -    tcp   2000
PARAM   -      -    tcp   9571
PARAM   -      -    tcp   9572
PARAM   -      -    tcp   10809
EOF.ltsp.macro

# Nagios macro.
cat > /etc/shorewall/macro.Nagios << EOF.nagios.macro
?FORMAT 2
#ACTION SOURCE DEST PROTO DPORT
PARAM   -      -    tcp   5666
EOF.nagios.macro

# NFS macro.
cat > /etc/shorewall/macro.NFS << EOF.nfs.macro
?FORMAT 2
#ACTION SOURCE DEST PROTO DPORT
PARAM   -      -    tcp   111
PARAM   -      -    tcp   2049
PARAM   -      -    tcp   4000
PARAM   -      -    tcp   4001
PARAM   -      -    tcp   4002
EOF.nfs.macro

# Usermin macro.
cat > /etc/shorewall/macro.Usermin << EOF.usermin.macro
?FORMAT 2
#ACTION SOURCE DEST PROTO DPORT
PARAM   -      -    tcp   20000
EOF.usermin.macro

# Create zones.
cat > /etc/shorewall/zones << EOF.zones
#ZONE  TYPE
fw     firewall
wan    ipv4
lan    ipv4
vpn    ipv4
EOF.zones

# Create interfaces.
cat << EOF.interfaces | column -t > /etc/shorewall/interfaces
#ZONE	INTERFACE	BROADCAST	OPTIONS
lan	$LAN_INTERFACE	detect	tcpflags,nosmurfs,routefilter
wan	$WAN_INTERFACE	detect	dhcp,tcpflags,nosmurfs,routefilter
vpn	tun+	detect
EOF.interfaces

# Create policy.
cat > /etc/shorewall/policy << EOF.policy
#SOURCE  DEST  POLICY  LOG
\$FW     all   ACCEPT
lan      wan   ACCEPT
vpn      lan   ACCEPT
wan      all   DROP    info
all      all   REJECT  info
EOF.policy

# Create masq.
cat << EOF.masq | column -t > /etc/shorewall/masq
#INTERFACE	SOURCE
$WAN_INTERFACE	$LAN_NETWORK_ADDRESS/$CIDR
EOF.masq

# Create tunnels.
cat << EOF.tunnels | column -t > /etc/shorewall/tunnels
#TYPE	ZONE	GATEWAY
openvpnserver:1194	wan	0.0.0.0/0
EOF.tunnels

# Create action.SSHKnock.
touch /etc/shorewall/action.SSHKnock

# Create actions.
cat > /etc/shorewall/actions << EOF.actions
#ACTION
SSHKnock
EOF.actions

# Create SSHKnock.
cat > /etc/shorewall/SSHKnock << EOF.sshknock
use Shorewall::Chains;

if ( \$level ) {
  log_rule_limit( \$level,
  \$chainref,
  'SSHKnock',
  'ACCEPT',
  '',
  \$tag,
  'add',
  '-p tcp --dport 22   -m recent --rcheck --name SSH ' );

  log_rule_limit( \$level,
  \$chainref,
  'SSHKnock',
  'DROP',
  '',
  \$tag,
  'add',
  '-p tcp ! --dport 22 ' );
}

add_rule( \$chainref, '-p tcp --dport 22 -m recent --rcheck --seconds 60 \
  --name SSH -j ACCEPT' );
# For security purposes, use a unique set of ports for port knocking.
# Reference shorewall.net/PortKnocking.html and soloport.com/iptables.html.
add_rule( \$chainref, '-p tcp --dport 1700 -m recent --name SSH \
  --remove -j DROP' );
add_rule( \$chainref, '-p tcp --dport 1701 -m recent --name SSH \
  --set    -j DROP' );
add_rule( \$chainref, '-p tcp --dport 1702 -m recent --name SSH \
  --remove -j DROP' );

1;
EOF.sshknock

# Shorewall startup.
sed -i "s|startup=0|startup=1|" /etc/default/shorewall
pgrep shorewall > /dev/null
if [ $? -eq 1 ]; then
  shorewall start
else
  shorewall restart
fi

################################################################################
#                                                                              #
# The following routine installs and configures GnuPG, which allows you to     #
# encrypt and sign your data and communication, features a versatile key       #
# management system as well as access modules for all kinds of public key      #
# directories.                                                                 #
#                                                                              #
# gnupg.org                                                                    #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing GnuPG."
echo ""

# Install GnuPG.
apt_function install $GNUPG

# Create GnuPG keys.
GPG_ID=$((
rngd -r /dev/urandom
gpg --gen-key --batch << EOF.gpg
Key-Type: RSA
Key-Length: 8192
Expire-Date: 7300
Name-Real: $SERVER_ADMIN
Name-Email: $EMAIL_ADDRESS
EOF.gpg
killall rngd
) 2>&1 | awk 'END { print }' | grep "gpg: key" | awk '{ print $3 }')

# Configure devscripts.conf for Debuild package signing.
sed -i "s|# DEBSIGN_KEYID=.*$|DEBSIGN_KEYID=$GPG_ID|
  s|# DEBUILD_LINTIAN_OPTS=.*$|DEBUILD_LINTIAN_OPTS=\"-i -I --show-overrides\"|" /etc/devscripts.conf

################################################################################
#                                                                              #
# The following routine installs and configures OpenSSL, which implements      #
# cryptographic functions and provides a Certificate Authority.                #
#                                                                              #
# openssl.org                                                                  #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing OpenSSL."
echo ""

# Install OpenSSL.
apt_function install $OPENSSL

# Configure openssl.cnf.
if [ ! -f /etc/ssl/openssl.cnf.orig ]; then 
  cp /etc/ssl/openssl.cnf /etc/ssl/openssl.cnf.orig
fi
grep -q "DNS:$FQDN" /etc/ssl/openssl.cnf || \
sed -ri "s|(default_bits\s+).+$|\1= 8192|
  s|365|7300|
  s|./demoCA|/etc/ssl|
  s|cacert.pem|ca-cert.pem|
  s|cakey.pem#|ca-key.pem #|
  /client, email, objsign/ a\nsCertType = client, server, email, objsign
  /email:copy/ a\subjectAltName \
= \"email:copy, DNS:$DYNAMICDNS_HOST, DNS:$FQDN\"" /etc/ssl/openssl.cnf

# Create CA directory structure.
mkdir -p /etc/ssl/certs
mkdir -p /etc/ssl/crl
mkdir -p /etc/ssl/newcerts
mkdir -p /etc/ssl/private
if [ -f /etc/ssl/index.txt ]; then
  rm -f /etc/ssl/index.txt
  touch /etc/ssl/index.txt
else
  touch /etc/ssl/index.txt
fi
if [ -f /etc/ssl/serial ]; then
  rm -f /etc/ssl/serial
  echo "01" > /etc/ssl/serial
fi

# Note: the CA expiration must be longer than the server certificate expiration
# (e.g., 7301 days for CA expiration and 7300 days for certificate expiration).
openssl req -new -keyout /etc/ssl/private/ca-key.pem \
  -passout pass:$ADMIN_PASSWORD \
  -out /etc/ssl/careq.pem -days 7301 \
  -subj "/C=$COUNTRY/ST=$STATE/O=$ORGANIZATION/OU=PKI/CN=$FQDN/\
emailAddress=$EMAIL_ADDRESS" 
openssl ca -batch -create_serial -out /etc/ssl/ca-cert.pem -days 7301 \
  -keyfile /etc/ssl/private/ca-key.pem -passin pass:$ADMIN_PASSWORD \
  -subj "/C=$COUNTRY/ST=$STATE/O=$ORGANIZATION/OU=PKI/\
CN=$FQDN/emailAddress=$EMAIL_ADDRESS" \
  -selfsign -infiles /etc/ssl/careq.pem

# Under rare circumstances, the CA root cert shows zero byte length and has to 
# be regenerated.
while [ ! -s /etc/ssl/ca-cert.pem ]; do
  openssl req -new -keyout /etc/ssl/private/ca-key.pem \
    -passout pass:$ADMIN_PASSWORD -out /etc/ssl/careq.pem -days 7301 \
    -subj "/C=$COUNTRY/ST=$STATE/O=$ORGANIZATION/OU=PKI/CN=$FDQN/\
emailAddress=$EMAIL_ADDRESS"
  openssl ca -batch -create_serial -out /etc/ssl/ca-cert.pem -days 7301 \
    -keyfile /etc/ssl/private/ca-key.pem -passin pass:$ADMIN_PASSWORD \
    -subj "/C=$COUNTRY/ST=$STATE/O=$ORGANIZATION/OU=PKI/CN=$FQDN/\
emailAddress=$EMAIL_ADDRESS" \
    -selfsign -infiles /etc/ssl/careq.pem
done
rm -f /etc/ssl/careq.pem

# Create ca-cert.pem hash value.
ln -fs /etc/ssl/ca-cert.pem /etc/ssl/`openssl x509 -noout -hash \
  -in /etc/ssl/ca-cert.pem`.0

# Set ca-key.pem permissions.
chmod 400 /etc/ssl/private/ca-key.pem

# Converting the CA certificate to x509 format is necessary when importing 
# the ca-cert into a client via http, e.g., http://rt1.example.corp/ca-cert.crt.
mkdir -p /var/www/html
openssl x509 -in /etc/ssl/ca-cert.pem -out /var/www/html/ca-cert.crt

# Create Certificate Revocation List (CRL).
echo "01" > /etc/ssl/crlnumber
openssl ca -gencrl -passin pass:$ADMIN_PASSWORD -out /etc/ssl/crl.pem
ln -fs /etc/ssl/crl.pem /etc/ssl/`openssl crl -noout -hash \
  -in /etc/ssl/crl.pem`.r0
ln -fs /etc/ssl/crl.pem /var/www/html/crl.pem
sed -i "s|nsCaRevocationUrl.*$\
|nsCaRevocationUrl\t\t http://$DYNAMICDNS_HOST/crl.pem|" /etc/ssl/openssl.cnf

# Create TLS certificate and key.
openssl req -new -nodes -keyout /etc/ssl/private/tls-key.pem \
  -out newreq.pem -subj "/C=$COUNTRY/ST=$STATE/O=$ORGANIZATION/\
OU=TLS/CN=$FQDN/emailAddress=$EMAIL_ADDRESS"
openssl ca -batch -out newcert.pem -passin pass:$ADMIN_PASSWORD -infiles newreq.pem
mv newcert.pem /etc/ssl/certs/tls-cert.pem
ln -fs /etc/ssl/certs/tls-cert.pem /etc/ssl/certs/`openssl x509 -noout \
  -hash -in /etc/ssl/certs/tls-cert.pem`.0  
rm -f newcert.pem newreq.pem
chmod 440 /etc/ssl/private/tls-key.pem

# Update AppArmor's profile for <abstractions/ssl_certs>.
if [ ! -f /etc/apparmor.d/abstractions/ssl_certs.orig ]; then 
  cp /etc/apparmor.d/abstractions/ssl_certs \
    /etc/apparmor.d/abstractions/ssl_certs.orig
fi
grep -q "/etc/ssl/ca-cert.pem" /etc/apparmor.d/abstractions/ssl_certs || \
cat >> /etc/apparmor.d/abstractions/ssl_certs << EOF.apparmor.ssl
  /etc/ssl/ca-cert.pem r,
  /etc/ssl/crl.pem r,
  /etc/pkcs11/modules/* r,
EOF.apparmor.ssl
systemctl restart apparmor

# Create Diffie-Hellman parameters.
openssl dhparam -out /etc/ssl/dh 8192

################################################################################
#                                                                              #
# The following routine installs and configures Apache, a robust,              #
# commercial-grade, featureful, and freely-available source code               #
# implementation of a HTTP (Web) server.                                       #
#                                                                              #
# apache.org                                                                   #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing Apache."
echo ""

# Install Apache.
apt_function install $APACHE $PHP

# Enable necessary Apache modules.
a2enmod expires rewrite ssl fcgid authnz_external passenger

# Enable mcrypt extension (phpMyAdmin Depends).
phpenmod mcrypt 

# Add ServerName to default sites.
if [ ! -f /etc/apache2/sites-available/000-default.conf.orig ]; then 
  cp /etc/apache2/sites-available/000-default.conf \
    /etc/apache2/sites-available/000-default.conf.orig
fi
sed -i "s|#ServerName www.example.com|ServerName $FQDN|" \
  /etc/apache2/sites-available/000-default.conf
if [ ! -f /etc/apache2/sites-available/default-ssl.conf.orig ]; then 
  cp /etc/apache2/sites-available/default-ssl.conf \
    /etc/apache2/sites-available/default-ssl.conf.orig
fi
grep -q "ServerName" /etc/apache2/sites-available/default-ssl.conf || \
  sed -i "/ServerAdmin/ a\ \n\t\tServerName $FQDN" \
    /etc/apache2/sites-available/default-ssl.conf

# Add External authentication.
grep -q "DefineExternalAuth" /etc/apache2/sites-available/000-default.conf || \
  sed -i "/ServerName $FQDN/ a\ \n\tDefineExternalAuth pwauth pipe \
/usr/sbin/pwauth" /etc/apache2/sites-available/000-default.conf
grep -q "DefineExternalAuth" /etc/apache2/sites-available/default-ssl.conf || \
  sed -i "/ServerName $FQDN/ a\ \n\t\tDefineExternalAuth pwauth pipe \
/usr/sbin/pwauth" /etc/apache2/sites-available/default-ssl.conf

# Add OpenSSL RewriteEngine.
grep -q "RewriteEngine On" /etc/apache2/sites-available/000-default.conf || \
  sed -i "/ServerName $FQDN/ a\ \n\tRewriteEngine On\n\tRewriteCond %{HTTPS} \
!=on\n\tRewriteRule ^/?(.*) https://%{SERVER_NAME}/\$1 [R,L]" \
  /etc/apache2/sites-available/000-default.conf

# Configure default-ssl.conf.
sed -i "s|/etc/ssl/certs/ssl-cert-snakeoil.pem|/etc/ssl/certs/tls-cert.pem|
  s|/etc/ssl/private/ssl-cert-snakeoil.key|/etc/ssl/private/tls-key.pem|
  s|#SSLCACertificateFile .*$|SSLCACertificateFile /etc/ssl/ca-cert.pem|
  s|#SSLCARevocationFile .*$|SSLCARevocationFile /etc/ssl/crl.pem|" \
    /etc/apache2/sites-available/default-ssl.conf
grep -q "SSLCARevocationCheck" \
  /etc/apache2/sites-available/default-ssl.conf || \
  sed -i "/SSLCARevocationFile/ a\ \t\tSSLCARevocationCheck chain" \
    /etc/apache2/sites-available/default-ssl.conf

# Enable default-ssl VirtualHost.
a2ensite default-ssl

# Configure ports.conf.
cat > /etc/apache2/ports.conf << EOF.ports.conf
Listen 80
<IfModule ssl_module>
  Listen 443
</IfModule>
EOF.ports.conf

# Add the www-data user to ssl-cert group so the daemon can read the TLS key.
usermod -aG ssl-cert www-data

# Reload configuration.
systemctl restart apache2

################################################################################
#                                                                              #
# The following routine installs and configures MySQL and phpMyAdmin.          #
#                                                                              #
# MySQL provides a relational database management system (RDBMS) that runs as  #
# a server providing multi-user access to a number of databases.               #
#                                                                              #
# phpMyAdmin provides a graphical user interface to manage MySQL.              #
#                                                                              #
# mysql.com                                                                    #
# phpmyadmin.net                                                               #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing MySQL."
echo ""

# Install MySQL.
apt_function install $MYSQL

# Configure bind address.
if [ ! -f /etc/mysql/my.cnf.orig ]; then 
  cp /etc/mysql/my.cnf /etc/mysql/my.cnf.orig
fi
sed -ri "s|(bind-address\s+).+$|\1= $LAN_IP_ADDRESS|" /etc/mysql/my.cnf
service mysql force-reload

# Set MySQL's root password.
mysqladmin --user=root password $ADMIN_PASSWORD

# Add MySQL's administrator account.
mysql --user=root --password=$ADMIN_PASSWORD << EOF.mysql_admin
GRANT ALL PRIVILEGES ON *.* TO 'admin'@'$LAN_IP_ADDRESS'\
  IDENTIFIED BY '$ADMIN_PASSWORD' WITH GRANT OPTION;
FLUSH PRIVILEGES; 
EOF.mysql_admin

# Change database backup compression.
sed -i "s|COMP=gzip|COMP=bzip2|" /etc/default/automysqlbackup

# Create a controluser for phpMyAdmin.
mysql --user=root --password=$ADMIN_PASSWORD << EOF.controluser
GRANT USAGE ON mysql.* TO 'pma'@'$LAN_IP_ADDRESS' IDENTIFIED BY '$ADMIN_PASSWORD';
GRANT SELECT (
  Host, User, Select_priv, Insert_priv, Update_priv, Delete_priv,
  Create_priv, Drop_priv, Reload_priv, Shutdown_priv, Process_priv,
  File_priv, Grant_priv, References_priv, Index_priv, Alter_priv,
  Show_db_priv, Super_priv, Create_tmp_table_priv, Lock_tables_priv,
  Execute_priv, Repl_slave_priv, Repl_client_priv
  ) ON mysql.user TO 'pma'@'$LAN_IP_ADDRESS';
GRANT SELECT ON mysql.db TO 'pma'@'$LAN_IP_ADDRESS';
GRANT SELECT ON mysql.host TO 'pma'@'$LAN_IP_ADDRESS';
GRANT SELECT (Host, Db, User, Table_name, Table_priv, Column_priv)
  ON mysql.tables_priv TO 'pma'@'$LAN_IP_ADDRESS';
GRANT SELECT, INSERT, UPDATE, DELETE ON phpmyadmin.* TO 'pma'@'$LAN_IP_ADDRESS';
EOF.controluser

# Disable phpMyAdmin's Suhosin warning.
grep -q "SuhosinDisableWarning" /etc/phpmyadmin/config.inc.php || \
cat >> /etc/phpmyadmin/config.inc.php << EOF.config.inc.php

/* Disable displayed warning on the main page when Suhosin is detected. */
\$cfg['SuhosinDisableWarning'] = TRUE;
EOF.config.inc.php

# Configure phpMyAdmin's Debconf.
if [ ! -f /etc/dbconfig-common/phpmyadmin.conf.orig ]; then 
  cp /etc/dbconfig-common/phpmyadmin.conf /etc/dbconfig-common/phpmyadmin.conf.orig
fi
sed -i "/dbc_dbpass=/ c\dbc_dbpass='$ADMIN_PASSWORD'
  /dbc_dbserver/ c\dbc_dbserver='$LAN_IP_ADDRESS'" \
    /etc/dbconfig-common/phpmyadmin.conf
if [ ! -f /etc/phpmyadmin/config-db.php.orig ]; then 
  cp /etc/phpmyadmin/config-db.php /etc/phpmyadmin/config-db.php.orig
fi
sed -i "/\$dbuser/ c\$dbuser='pma';
  /\$dbpass=/ c\$dbpass='$ADMIN_PASSWORD';
  /\$dbserver/ c\$dbserver='$LAN_IP_ADDRESS';" /etc/phpmyadmin/config-db.php
chmod 640 /etc/dbconfig-common/phpmyadmin.conf /etc/phpmyadmin/config-db.php
chown root:www-data /etc/dbconfig-common/phpmyadmin.conf \
  /etc/phpmyadmin/config-db.php

# Configure phpMyAdmin's apache.conf.
if [ ! -f /etc/phpmyadmin/apache.conf.orig ]; then 
  cp /etc/phpmyadmin/apache.conf /etc/phpmyadmin/apache.conf.orig
fi
sed -i "/Require ip/ c\Require ip 127.0.0.0/8 $LAN_NETWORK_ADDRESS/$CIDR" \
  /etc/phpmyadmin/apache.conf
ln -fs /etc/phpmyadmin/apache.conf /etc/apache2/conf-available/phpmyadmin.conf
a2enconf phpmyadmin

# Add the MySQL user to ssl-cert group so the daemon can read the TLS key.
usermod -aG ssl-cert mysql

# Reload configuration.
systemctl restart mysql 
systemctl restart apache2

################################################################################
#                                                                              #
# The following routine installs and configures ClamAV, which provides an      #
# effective malware scanner.                                                   #
#                                                                              #
# clamav.net                                                                   #
# wbmclamav.labs.libre-entreprise.org                                          #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing malware scanner."
echo ""

# Install malware scanner.
apt_function install $CLAMAV

# Configure clamd.conf.
if [ ! -f /etc/clamav/clamd.conf.orig ]; then 
  cp /etc/clamav/clamd.conf /etc/clamav/clamd.conf.orig
fi
sed -i "s|DetectPUA false|DetectPUA true|
  s|HeuristicScanPrecedence false|HeuristicScanPrecedence true|
  s|StructuredDataDetection false|StructuredDataDetection true|" \
    /etc/clamav/clamd.conf

# Enable SafeBrowsing.
if [ ! -f /etc/clamav/freshclam.conf.orig ]; then 
  cp /etc/clamav/freshclam.conf /etc/clamav/freshclam.conf.orig
fi
grep -q "SafeBrowsing yes" /etc/clamav/freshclam.conf || \
  echo "SafeBrowsing yes" >> /etc/clamav/freshclam.conf

# Update malware signatures.
clear
echo ""
echo "Please wait for FreshClam to update ClamAV malware signatures."
echo ""
freshclam

# Configure group.
chgrp clamav /run/clamav

# Reload configuration.
systemctl restart clamav-daemon
systemctl restart clamav-freshclam

################################################################################
#                                                                              #
# The following routine installs and configures Privoxy, Squid, SquidClamAV,   #
# SquidGuard, and SquidGuarMgr.                                                #
#                                                                              #
# Privoxy filters unwanted advertisements and Internet junk that suck up       #
# precious bandwidth.                                                          #
#                                                                              #
# Squid creates a cache of frequently accessed web pages, which improves       #
# performance and helps conserve bandwidth.                                    #
#                                                                              #
# SquidClamAV stops malware before it reaches your workstations.               #
#                                                                              #
# SquidGuard provides an effective Internet content filter.                    #
#                                                                              #
# SquidGuardMgr provides a graphical user interface to manage SquidClamAV and  #
# SquidGuard.                                                                  #
#                                                                              #
# privoxy.org                                                                  #
# shallalist.de                                                                #
# squidguard.org                                                               #
# squid-cache.org                                                              #
# squidclamav.darold.net/index.html                                            #
# sourceforge.net/projects/squidguardmgr                                       #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing Proxy."
echo ""

# Install Proxy.
apt_function install $PROXY

# Configure squid.conf.
if [ ! -f /etc/squid/squid.conf.orig ]; then 
  cp /etc/squid/squid.conf /etc/squid/squid.conf.orig
fi
sed -i "s|#acl localnet src 192.168.0.0/16\
|acl localnet src $LAN_NETWORK_ADDRESS/$CIDR|
  s|http_port 3128|http_port 3128 intercept|
  /http_access allow localnet/ s|^#||" /etc/squid/squid.conf
grep -q "# SquidClamAV configuration" /etc/squid/squid.conf || \
cat >> /etc/squid/squid.conf << EOF.squid.conf

# SquidClamAV configuration.
url_rewrite_program /usr/bin/squidGuard
# More than two processes will bog down system if SquidGuard is overloaded.
url_rewrite_children 2 
url_rewrite_access allow all
icap_enable on
icap_send_client_ip on
icap_send_client_username on
icap_client_username_encode off
icap_client_username_header X-Authenticated-User
icap_preview_enable on
icap_preview_size 1024
icap_service service_req reqmod_precache bypass=1 icap://127.0.0.1:1344\
/squidclamav
adaptation_access service_req allow all
icap_service service_resp respmod_precache bypass=1 icap://127.0.0.1:1344\
/squidclamav
adaptation_access service_resp allow all

# Define Privoxy as parent proxy (without ICP).
cache_peer 127.0.0.1 parent 8118 7 no-query

# Define ACL for protocol FTP.
acl ftp proto FTP

# Do not forward FTP requests to Privoxy.
always_direct allow ftp

# Forward all the rest to Privoxy.
never_direct allow all
EOF.squid.conf

# Create Squid's cache directories.
systemctl stop squid
squid -z
systemctl start squid

# Install SquidClamAV.
apt_function install $SQUIDCLAMAV_DEPENDS
wget_tar_function $SQUIDCLAMAV squidclamav
cd /usr/local/src/squidclamav-*
sh configure --prefix=/usr --sysconfdir=/etc --datadir=/usr/share --with-c-icap
make && make install
cd ..
rm -rf squidclamav-*

# Configure squidclamav.conf.
if [ ! -f /etc/c-icap/squidclamav.conf.orig ]; then 
  cp /etc/c-icap/squidclamav.conf /etc/c-icap/squidclamav.conf.orig
fi
sed -i "s|redirect http://.*$|redirect http://$LAN_IP_ADDRESS/cgi-bin/clwarn.cgi|
  s|safebrowsing 0|safebrowsing 1|" /etc/c-icap/squidclamav.conf

# Configure c-icap.conf.
if [ ! -f /etc/c-icap/c-icap.conf.orig ]; then 
  cp /etc/c-icap/c-icap.conf /etc/c-icap/c-icap.conf.orig
fi
sed -i "s|ServerAdmin .*$|ServerAdmin $EMAIL_ADDRESS|
  s|ServerName YourServerName|ServerName $FQDN|" /etc/c-icap/c-icap.conf
grep -q "# Enable SquidClamAV" /etc/c-icap/c-icap.conf || \
cat >> /etc/c-icap/c-icap.conf << EOF.c-icap.conf

# Enable SquidClamAV.
Service squidclamav squidclamav.so
EOF.c-icap.conf

# Setup clwarn.cgi.
chgrp www-data /usr/lib/cgi-bin
cp -f /usr/libexec/squidclamav/clwarn.cgi.en_EN /usr/lib/cgi-bin/clwarn.cgi

# Install SquidGuardMgr.
wget_tar_function $SQUIDGUARDMGR squidguardmgr
cd /usr/local/src/squidguardmgr-*

# Patches for upstream source code.
patch -ful -d cgi-bin/lang/en_US << EOF.acl.html
--- acl.html	2015-01-23 08:14:48.000000000 -0800
+++ acl.html	2015-03-25 21:20:06.951992676 -0700
@@ -7,7 +7,7 @@
 </p>
 
 <p>
-The <b>Deny ip addresses in URLs</b> can be used to enforce the use of \
domainnames over IP addresses in the host part of URLs.
+The <b>Deny IP addresses in URLs</b> can be used to enforce the use of \
domainnames over IP addresses in the host part of URLs.
 </p>
 
 <p>
EOF.acl.html

patch -ful -d cgi-bin/lang/en_US << EOF.globals.html
--- globals.html	2015-01-23 08:14:48.000000000 -0800
+++ globals.html	2015-03-25 21:23:28.415983694 -0700
@@ -2,7 +2,7 @@
 
 <h4>Log directory:</h4>
 <pre>
-	This mandatory parameter is used to define the full path where logi
+	This mandatory parameter is used to define the full path where log
 	files will be written.
 	By default: /usr/local/squidGuard/logs
 </pre>
EOF.globals.html

patch -ful -d cgi-bin/lang/en_US << EOF.sources.html
--- sources.html        2020-01-06 14:17:41.771993324 -0800
+++ sources.html        2020-06-24 10:50:47.026246750 -0700
@@ -18,9 +18,9 @@
      The file format is simply addresses and/or networks separated by a newline as above.
 </pre>

-<h4>Search Ip in LDAP:</h4>
+<h4>Search IP in LDAP:</h4>
 <pre>
-    Used to define a ldap query in order to extract an Ip address from a LDAP server (or an Active Directory). 
+    Used to define a LDAP query in order to extract an IP address from a LDAP server (or an Active Directory). 
     Use '%s' to reference the IP in your LDAP URL.

 Example:
@@ -53,7 +53,7 @@

 <h4>Search Users in LDAP:</h4>
 <pre>
-    Used to define a ldap query in order to extract a user name from a LDAP server (or an Active Directory).
+    Used to define a LDAP query in order to extract a user name from a LDAP server (or an Active Directory).
     Use '%s' to reference the username in your LDAP URL.

 example:
EOF.sources.html

# Configure SquidGuardMgr's Apache directory.
mkdir -p /var/www/html/squidguardmgr
cp -au {cgi-bin/*,htdocs/*} /var/www/html/squidguardmgr
cp -u squidguardmgr.conf.sample /var/www/html/squidguardmgr/squidguardmgr.conf
chown -R proxy:proxy /var/www/html/squidguardmgr
sed -i "/CONF_FILE/ c\CONF_FILE     /etc/squidguard/squidGuard.conf
  /SQUIDCLAMAV/ c\SQUIDCLAMAV     c-icap
  /C_ICAP_SOCKET/ c\C_ICAP_SOCKET     /var/run/c-icap/c-icap.pid
  /SC_CONF_FILE/ c\SC_CONF_FILE     /etc/c-icap/squidclamav.conf
  /SQUID_WRAPPER/ c\SQUID_WRAPPER     /var/www/html/squidguardmgr\
/squid_wrapper" /var/www/html/squidguardmgr/squidguardmgr.conf
sed -i "s|/var/lib/squidguard/db|/var/lib/squidGuard/db|
  s|/usr/local/bin/squidGuard|/usr/bin/squidGuard|
  s|/usr/local/squidGuard/squidguard.conf|/etc/squidguard/squidGuard.conf|
  s|/etc/squidclamav.conf|/etc/c-icap/squidclamav.conf|
  s|/var/www/squidguardmgr/squid_wrapper|/var/www/html/squidguardmgr\
/squid_wrapper|" /var/www/html/squidguardmgr/squidguardmgr.cgi
cc -o squid_wrapper/squid_wrapper squid_wrapper/squid_wrapper.c
chown root:root squid_wrapper/squid_wrapper
chmod 2755 squid_wrapper/squid_wrapper
cp -u squid_wrapper/squid_wrapper /var/www/html/squidguardmgr
cd ..
rm -rf squidguardmgr-*

# Configure squidguardmgr.conf.
cat > /etc/apache2/conf-available/squidguardmgr.conf << EOF.squidguardmgr.conf
Alias /squidguardmgr /var/www/html/squidguardmgr/
<Directory /var/www/html/squidguardmgr/>
  Options +ExecCGI
  AddHandler cgi-script .cgi
  DirectoryIndex squidguardmgr.cgi
  <RequireAll>
    Require ip 127.0.0.0/8 $LAN_NETWORK_ADDRESS/$CIDR
    SSLRequireSSL
    AuthName "SquidGuardMgr Login"
    AuthType Basic
    AuthBasicProvider external
    AuthExternal pwauth
    Require valid-user
  </RequireAll>
</Directory>
EOF.squidguardmgr.conf
a2enconf squidguardmgr.conf

# Create squidGuard.conf.
if [ ! -f /etc/squidguard/squidGuard.conf.orig ]; then 
  mv /etc/squidguard/squidGuard.conf /etc/squidguard/squidGuard.conf.orig
fi
cat > /etc/squidguard/squidGuard.conf << EOF.squidGuard.conf
dbhome /var/lib/squidguard/db
logdir /var/log/squid

dest whitelist {
  domainlist  WHITELIST/domains
}

dest warez {
  domainlist  BLACKLIST/warez/domains
  urllist     BLACKLIST/warez/urls
}

dest porn {
  domainlist  BLACKLIST/porn/domains
  urllist     BLACKLIST/porn/urls
}

dest spyware {
  domainlist  BLACKLIST/spyware/domains
  urllist     BLACKLIST/spyware/urls
}

dest violence {
  domainlist  BLACKLIST/violence/domains
  urllist     BLACKLIST/violence/urls
}

acl {
  default {
  pass whitelist !in-addr all
  redirect http://$LAN_IP_ADDRESS/squidguardmgr/blocked_website.html
  }
}
EOF.squidGuard.conf
chown -R www-data:proxy /etc/squidguard/squidGuard.conf
chown -R c-icap:proxy /var/log/squidguard/squidGuard.log

# Create "blocked website" redirect HTML page.
cat > /var/www/html/squidguardmgr/blocked_website.html << EOF.blocked_website.html
<html>
  <body>
    <div align="center">
      <H1><font color="#FF0000">The website you attempted to access is blocked \
because it contains restricted content.</font></H1>
      <p>If you feel this message is in error, \
please contact $EMAIL_ADDRESS.</p>
    </div>
  </body>
</html>
EOF.blocked_website.html

# Update Shalla Blacklists.
$WGET $SHALLA_BLACKLISTS
mkdir -p /var/lib/squidguard/db/WHITELIST
touch /var/lib/squidguard/db/WHITELIST/domains
tar xzf shallalist.tar.gz -C /var/lib/squidguard/db
mv -f /var/lib/squidguard/db/BL /var/lib/squidguard/db/BLACKLIST
while [ $? -ne 0 ]; do
  rm -f shallalist.tar.gz
  $WGET $SHALLA_BLACKLISTS
  tar xzf shallalist.tar.gz -C /var/lib/squidguard/db
  mv -f /var/lib/squidguard/db/BL /var/lib/squidguard/db/BLACKLIST
done
rm -f shallalist.tar.gz

# Create new .db files from urls/domains files.
squidGuard -dCP all
chown -R proxy /var/lib/squidguard/db
find /var/lib/squidguard/db -type f -exec chmod 644 {} \;
find /var/lib/squidguard/db -type d -exec chmod 755 {} \;

# Create Shalla Blacklist updater script.
cat > /usr/local/bin/shalla_updater.sh << EOF.shalla_updater.sh
#!/bin/bash
if [ ! -d /var/lib/squidguard/tmp ]; then
  mkdir -p /var/lib/squidguard/tmp
fi

cd /var/lib/squidguard/tmp

NEW=0
if [ -e /var/lib/squidguard/tmp/shallalist.tar.gz ]; then
  curl -o /var/lib/squidguard/tmp/shallalist.tar.gz.md5 \
    $SHALLA_BLACKLISTS.md5
  md5sum --status -c *.md5

  if [ \$? -ne 0 ]; then
    unlink shallalist.tar.gz
    $WGET $SHALLA_BLACKLISTS
    NEW=1
  fi
else
  curl -o /var/lib/squidGuard/tmp/shallalist.tar.gz.md5 \
    $SHALLA_BLACKLISTS.md5
  $WGET $SHALLA_BLACKLISTS
  NEW=1
fi

if [ \$NEW -eq 1 ]; then
  md5sum --status -c *.md5
  if [ \$? -eq 0 ]; then
    tar -zxf shallalist.tar.gz -C /var/lib/squidguard/db
    mv -f /var/lib/squidguard/db/BL /var/lib/squidguard/db/BLACKLIST
    squidGuard -dCP all
    chown --recursive squid /var/lib/squidguard/db
    find /var/lib/squidguard/db -type f -exec chmod 644 {} \;
    find /var/lib/squidguard/db -type d -exec chmod 755 {} \;
    squid -k reconfigure
  fi
fi
EOF.shalla_updater.sh
chmod 700 /usr/local/bin/shalla_updater.sh
echo "@daily root /usr/local/bin/shalla_updater.sh" > /etc/cron.d/shalla_updater

# Configure Privoxy's configuration.
if [ ! -f /etc/privoxy/config.orig ]; then 
  cp /etc/privoxy/config /etc/privoxy/config.orig
fi
sed -i "s|#admin-address .*$|admin-address $EMAIL_ADDRESS|
  s|enable-proxy-authentication-forwarding 0\
|enable-proxy-authentication-forwarding 1|" /etc/privoxy/config
chown privoxy /etc/privoxy/*.action

# Configure Privoxy's match-all.action.
if [ ! -f /etc/privoxy/match-all.action.orig ]; then 
  cp /etc/privoxy/match-all.action /etc/privoxy/match-all.action.orig
fi
sed -i "s|+set-image-blocker{pattern}|+set-image-blocker{blank}|" \
  /etc/privoxy/match-all.action

# Add the proxy user to www-data group so the daemon can read the http.keytab.
usermod -aG proxy www-data

# Reload configuration.
systemctl restart apache2
systemctl restart privoxy

# c-icap startup.
sed -i "s|START=no|START=yes|" /etc/default/c-icap
pgrep c-icap > /dev/null
if [ $? -eq 1 ]; then
  systemctl start c-icap
else
  systemctl restart c-icap
fi

################################################################################
#                                                                              #
# The following routine installs and configures Samba, an open-source          #
# software suite that provides seamless file and print services to CIFS        #
# clients.                                                                     #
#                                                                              #
# samba.org                                                                    #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing Samba."
echo ""

# Install Samba.
apt_function install $CUPS $SAMBA

# Configure smb.conf.
if [ ! -f /etc/samba/smb.conf.orig ]; then 
  cp /etc/samba/smb.conf /etc/samba/smb.conf.orig
fi
cat > /etc/samba/smb.conf << EOF.smb.conf
[global]
  workgroup = $WORKGROUP
  server role = standalone
  strict allocate = yes
  hosts allow = 127.0.0.0/8 $LAN_NETWORK_ADDRESS/$CIDR 10.8.0.0/24 
  interfaces  = 127.0.0.0/8 $LAN_NETWORK_ADDRESS/$CIDR 10.8.0.0/24

[printers]
  comment = All printers.
  path = /var/spool/samba
  browseable = no
  printable = yes

[print\$]
  comment = Printer drivers.
  path = /var/lib/samba/printers
  browseable = no

[sharedDocs]
  comment = Public share.
  path = /home/sharedDocs
  writeable = yes
EOF.smb.conf

# Create directory structure.
mkdir -p -m 1777 /var/spool/samba
mkdir -p -m 777 /home/sharedDocs
mkdir -p /var/lib/samba/printers/COLOR
mkdir -p /var/lib/samba/printers/IA64
mkdir -p /var/lib/samba/printers/W32ALPHA
mkdir -p /var/lib/samba/printers/W32MIPS
mkdir -p /var/lib/samba/printers/W32PPC
mkdir -p /var/lib/samba/printers/W32X86
mkdir -p /var/lib/samba/printers/WIN40
mkdir -p /var/lib/samba/printers/x64

# Reload configuration.
systemctl restart smbd 

################################################################################
#                                                                              #
# The following routine installs and configures NFS (Network File System),     #
# which allows for fast, seamless sharing of files across a network.           #
#                                                                              #
# ietf.org/rfc/rfc3530.txt                                                     #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing NFS."
echo ""

# Install NFS.
apt_function install $NFS

# Add NFS exports.
if [ ! -f /etc/fstab.orig ]; then 
  cp /etc/fstab /etc/fstab.orig
fi
mkdir -p /nfs/home
echo "/nfs $LAN_NETWORK_ADDRESS/$CIDR(rw)" > /etc/exports
echo "/nfs/home $LAN_NETWORK_ADDRESS/$CIDR(rw)" >> /etc/exports
mount --bind /home /nfs/home
grep -q "/nfs/home" /etc/fstab || \
  echo "/home /nfs/home none bind 0 0" >> /etc/fstab
exportfs -a

# Configure idmapd.conf.
if [ ! -f /etc/idmapd.conf.orig ]; then 
  cp /etc/idmapd.conf /etc/idmapd.conf.orig
fi
sed -i "s|Domain = .*$|Domain = $LAN_DOMAIN|" /etc/idmapd.conf

# nfs-kernel-server startup.
pgrep nfsd > /dev/null
if [ $? -eq 1 ]; then
  systemctl start nfs-kernel-server
else
  systemctl restart nfs-kernel-server
fi

################################################################################
#                                                                              #
# The following routine installs and configures ProFTPD, a                     # 
# highly-configurable, secure FTP server.                                      #
#                                                                              #
# proftpd.org                                                                  #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing ProFTPD."
echo ""

# Install ProFTPD.
apt_function install $PROFTPD

# Configure proftpd.conf.
if [ ! -f /etc/proftpd/proftpd.conf.orig ]; then 
  cp /etc/proftpd/proftpd.conf /etc/proftpd/proftpd.conf.orig
fi
sed -ri "s|(ServerName\s+).+$|\1\"$FQDN\"|
  /tls.conf/ s|^#||
  s|(PersistenPasswd\s+).+$|\1on|" /etc/proftpd/proftpd.conf

# Configure tls.conf.
if [ ! -f /etc/proftpd/tls.conf.orig ]; then 
  cp /etc/proftpd/tls.conf /etc/proftpd/tls.conf.orig
fi
sed -ri "s|(TLSRSACerfificateFile\s+).+$|\1/etc/ssl/certs/tls-cert.pem|
  s|(TLSRSACerfificateKeyFile\s+).+$|\1/etc/ssl/certs/tls-key.pem|" \
    /etc/proftpd/tls.conf

# Reload configuration.
systemctl restart proftpd

################################################################################
#                                                                              #
# The following routine installs and configures Munin, a networked resource    #
# monitoring tool that can help analyze resource trends and performance        #
# problems.                                                                    #
#                                                                              #
# munin-monitoring.org                                                         #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing Munin."
echo ""

# Install Munin.
apt_function install $MUNIN_DEPENDS $MUNIN

# Install plugins.
rm -f /etc/munin/plugins/*
ln -fs /usr/share/munin/plugins/load /etc/munin/plugins
ln -fs /usr/share/munin/plugins/memory /etc/munin/plugins
ln -fs /usr/share/munin/plugins/cpu* /etc/munin/plugins
ln -fs /usr/share/munin/plugins/diskstat* /etc/munin/plugins
ln -fs /usr/share/munin/plugins/smart* /etc/munin/plugins
ln -fs /usr/share/munin/plugins/fw* /etc/munin/plugins
ln -fs /usr/share/munin/plugins/tcp /etc/munin/plugins
ln -fs /usr/share/munin/plugins/apache* /etc/munin/plugins
ln -fs /usr/share/munin/plugins/asterisk* /etc/munin/plugins
ln -fs /usr/share/munin/plugins/bind* /etc/munin/plugins
ln -fs /usr/share/munin/plugins/dhcpd* /etc/munin/plugins
ln -fs /usr/share/munin/plugins/mysql* /etc/munin/plugins
ln -fs /usr/share/munin/plugins/nfsd* /etc/munin/plugins
ln -fs /usr/share/munin/plugins/openvpn /etc/munin/plugins
ln -fs /usr/share/munin/plugins/postfix* /etc/munin/plugins
ln -fs /usr/share/munin/plugins/samba /etc/munin/plugins
ln -fs /usr/share/munin/plugins/snort* /etc/munin/plugins
ln -fs /usr/share/munin/plugins/squid* /etc/munin/plugins

# Configure munin.conf.
if [ ! -f /etc/munin/munin.conf.orig ]; then 
  cp /etc/munin/munin.conf /etc/munin/munin.conf.orig
fi
sed -i "s|\[localhost.localdomain\]|\[$FQDN\]|
  s|address 127.0.0.1|address $LAN_IP_ADDRESS|" /etc/munin/munin.conf

# Configure munin-node.conf.
if [ ! -f /etc/munin/munin-node.conf.orig ]; then 
  cp /etc/munin/munin-node.conf /etc/munin/munin-node.conf.orig
fi
sed -i "s|# cidr_allow 192.*$|cidr_allow $LAN_NETWORK_ADDRESS/$CIDR|
  s|#host_name localhost.localdomain|host_name $FQDN|
  s|host \*|host $LAN_IP_ADDRESS|" /etc/munin/munin-node.conf

# Configure apache.conf.
if [ ! -f /etc/munin/apache.conf.orig ]; then 
  cp /etc/munin/apache.conf /etc/munin/apache.conf.orig
fi
sed -i "/Require ip/ c\Require ip 127.0.0.0/8 $LAN_NETWORK_ADDRESS/$CIDR
    /<RequireAll>/ a\    SSLRequireSSL/n\
    AuthName \"Munin Login\"/n\
    AuthType Basic/n\
    AuthBasicProvider external/n\
    AuthExternal pwauth/n\
    Require valid-user" /etc/munin/apache.conf

# Reload configuration.
systemctl restart apache2
systemctl restart munin-node

################################################################################
#                                                                              #
# The following routine installs and configures Nagios, a powerful monitoring  #
# system that enables organizations to identify and resolve IT infrastructure  #
# problems before they affect critical business processes.                     #
#                                                                              #
# nagios.org                                                                   #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing Nagios."
echo ""

# Install Nagios.
apt_function install $NAGIOS

# Backup original configuration to prepare for new custom configuration.
mv -u /etc/nagios4/conf.d /etc/nagios4/conf.d.orig
mkdir -p /etc/nagios4/conf.d

# Configure cgi.cfg.
if [ ! -f /etc/nagios4/cgi.cfg.orig ]; then 
  cp /etc/nagios4/cgi.cfg /etc/nagios4/cgi.cfg.orig
fi
sed -i "s|use_ssl_authentication=0|use_ssl_authentication=1|
  s|nagiosadmin|admin|
  s|use_authentication=1|use_authentication=0|" /etc/nagios4/cgi.cfg

# Configure apache2.conf.
if [ ! -f /etc/nagios4/apache2.conf.orig ]; then 
  cp /etc/nagios4/apache2.conf /etc/nagios4/apache2.conf.orig
fi
sed -i "/Require ip/ c\Require ip 127.0.0.0/8 $LAN_NETWORK_ADDRESS/$CIDR
    /<RequireAll>/ a\    SSLRequireSSL/n\
    AuthName \"Nagios Login\"/n\
    AuthType Basic/n\
    AuthBasicProvider external/n\
    AuthExternal pwauth/n\
    Require valid-user" /etc/nagios4/apache2.conf

# Configure hosts.cfg.
cat > /etc/nagios4/conf.d/hosts.cfg << EOF.hosts.cfg
# Host definitions.
define host {
  host_name              $HOSTNAME
  alias                  Ubuntu Router
  address                $LAN_IP_ADDRESS
  max_check_attempts     5
  check_period           24x7
  check_command          check_host_status
  contacts               root
  contact_groups         admins
  notification_interval  30
  notification_period    24x7
}
EOF.hosts.cfg

# Configure contacts.cfg.
cat > /etc/nagios4/conf.d/contacts.cfg << EOF.contacts.cfg
define contact {
  host_notifications_enabled     1
  service_notifications_enabled  1
  contact_name                   root
  service_notification_period    24x7
  host_notification_period       24x7
  service_notification_options   w,u,c,r
  host_notification_options      d,r
  service_notification_commands  notify-service-by-email
  host_notification_commands     notify-host-by-email
  email                          root@localhost
}

define contactgroup {
  contactgroup_name  admins
  alias              Nagios Administrators
  members            root
}
EOF.contacts.cfg

# Configure extinfo.cfg.
cat > /etc/nagios4/conf.d/extinfo.cfg << EOF.extinfo.cfg
# Extended Host and Service Information.
define hostextinfo {
  notes            Ubuntu Linux servers
  icon_image       base/Ubuntu.png
  icon_image_alt   Ubuntu Linux
  vrml_image       ubuntu.png
  statusmap_image  base/ubuntu.gd2
}
EOF.extinfo.cfg

# Configure timeperiods.cfg.
cat > /etc/nagios4/conf.d/timeperiods.cfg << EOF.timeperiods.cfg
# This defines a timeperiod where all times are valid for checks, 
# notifications, etc.
define timeperiod {
  timeperiod_name  24x7
  alias            24 Hours A Day, 7 Days A Week
  sunday           00:00-24:00
  monday           00:00-24:00
  tuesday          00:00-24:00
  wednesday        00:00-24:00
  thursday         00:00-24:00
  friday           00:00-24:00
  saturday         00:00-24:00
}
EOF.timeperiods.cfg

# Configure check_commands.cfg.
chmod u+s  /usr/lib/nagios/plugins/check_dhcp # Must be run as setuid root.
chmod u+s  /usr/lib/nagios/plugins/check_host 
cat > /etc/nagios4/conf.d/check_commands.cfg << EOF.check_commands.cfg
define command {
  command_name check_host_status
  command_line /usr/lib/nagios/plugins/check_host -H localhost
}

define command {
  command_name check_ntp_status
  command_line /usr/lib/nagios/plugins/check_ntp_time -H pool.ntp.org
}

define command {
  command_name check_dhcp_status
  command_line /usr/lib/nagios/plugins/check_dhcp -u
}

define command {
  command_name check_smb_status
  command_line /usr/lib/nagios/plugins/check_disk_smb -H localhost -s \$ARG1\$
}

define command {
  command_name check_mysql_status
  command_line /usr/lib/nagios/plugins/check_mysql -H localhost -u \$ARG1\$ \
-p \$ARG2\$
}

define command {
  command_name check_nagios_status
  command_line /usr/lib/nagios/plugins/check_nagios -e \$ARG1\$ -F \$ARG2\$ \
-C \$ARG3\$
}
EOF.check_commands.cfg

# Configure services.cfg.
cat > /etc/nagios4/conf.d/services.cfg << EOF.services.cfg
# Check that hosts are running.
define service {
  host_name              $HOSTNAME
  service_description    Host Service
  check_command          check_host_status
  max_check_attempts     5
  check_interval         5
  retry_interval         3
  check_period           24x7	
  notification_interval  30
  notification_period    24x7
  contacts               root
  contact_groups         admins	
}

# Check that SMB service is running.
define service {
  host_name              $HOSTNAME
  service_description    SMB Service
  check_command          check_smb_status!sharedDocs
  max_check_attempts     5
  check_interval         5
  retry_interval         3
  check_period           24x7	
  notification_interval  30
  notification_period    24x7
  contacts               root
  contact_groups         admins	
}

# Check that DNS service is running.
define service {
  host_name              $HOSTNAME
  service_description    DNS Service
  check_command          check_dns
  max_check_attempts     5
  check_interval         5
  retry_interval         3
  check_period           24x7	
  notification_interval  30
  notification_period    24x7
  contacts               root
  contact_groups         admins	
}

# Check that DHCP service is running.
define service {
  host_name              $HOSTNAME
  service_description    DHCP Service
  check_command          check_dhcp_status
  max_check_attempts     5
  check_interval         5
  retry_interval         3
  check_period           24x7	
  notification_interval  30
  notification_period    24x7
  contacts               root
  contact_groups         admins	
}

# Check that FTP service is running.
define service {
  host_name              $HOSTNAME
  service_description    FTP Service
  check_command          check_ftp
  max_check_attempts     5
  check_interval         5
  retry_interval         3
  check_period           24x7	
  notification_interval  30
  notification_period    24x7
  contacts               root
  contact_groups         admins	
}

# Check that Apache service is running.
define service {
  host_name              $HOSTNAME
  service_description    HTTP Service
  check_command          check_http
  max_check_attempts     5
  check_interval         5
  retry_interval         3
  check_period           24x7	
  notification_interval  30
  notification_period    24x7
  contacts               root
  contact_groups         admins	
}

# Check that NTP service is running.
define service {
  host_name              $HOSTNAME
  service_description    NTP Service
  check_command          check_ntp_status
  max_check_attempts     5
  check_interval         5
  retry_interval         3
  check_period           24x7	
  notification_interval  30
  notification_period    24x7
  contacts               root
  contact_groups         admins	
}

# Check that SMTP service is running.
define service {
  host_name              $HOSTNAME
  service_description    SMTP Service
  check_command          check_smtp
  max_check_attempts     5
  check_interval         5
  retry_interval         3
  check_period           24x7	
  notification_interval  30
  notification_period    24x7
  contacts               root
  contact_groups         admins	
}

# Check that MySQL service is running.
define service {
  host_name              $HOSTNAME
  service_description    MYSQL Service
  check_command          check_mysql_status!root!$ADMIN_PASSWORD
  max_check_attempts     5
  check_interval         5
  retry_interval         3
  check_period           24x7	
  notification_interval  30
  notification_period    24x7
  contacts               root
  contact_groups         admins	
}

# Check that Nagios service is running.
define service {
  host_name              $HOSTNAME
  service_description    Nagios Service
  check_command          check_nagios_status!5!/var/log/nagios4/nagios.log!\
/usr/sbin/nagios4
  max_check_attempts     5
  check_interval         5
  retry_interval         3
  check_period           24x7	
  notification_interval  30
  notification_period    24x7
  contacts               root
  contact_groups         admins	
}
EOF.services.cfg
chmod 600 /etc/nagios4/conf.d/services.cfg
chown nagios /etc/nagios4/conf.d/services.cfg

# Reload configuration.
systemctl restart nagios4

################################################################################
#                                                                              #
# The following routine installs and configures DDclient, a Perl client used   #
# to update dynamic DNS entries for accounts on Dynamic DNS Network Services'  #
# free DNS service.                                                            #
#                                                                              #
# sourceforge.net/apps/trac/ddclient                                           #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing DDclient."
echo ""

# Install DDclient.
apt_function install $DDCLIENT

# Configure ddclient.conf.
if [ ! -f /etc/ddclient.conf.orig ]; then 
  mv /etc/ddclient.conf /etc/ddclient.conf.orig
fi
cat > /etc/ddclient.conf << EOF.ddclient
daemon=300                        
syslog=yes                        
mail=root                         
mail-failure=root                 
pid=/run/ddclient.pid         
ssl=yes                           
use=if, if=$LAN_INTERFACE         
login=$EMAIL_ADDRESS            # Register login with dnsdynamic.org.
password=$ADMIN_PASSWORD       # Register password with dnsdynamic.org.
server=www.dnsdynamic.org, \ 
protocol=dyndns2 \
$DYNAMICDNS_HOST             # Register DYNAMICDNS_HOST with dnsdynamic.org.
EOF.ddclient
chmod 600 /etc/ddclient.conf
chown root:root /etc/ddclient.conf

# Reload configuration.
systemctl restart ddclient

################################################################################
#                                                                              #
# The following routine installs and configures OpenVPN, which implements      #
# virtual private network (VPN) techniques for creating secure point-to-point  #
# or site-to-site connections in routed or bridged configurations and remote   #
# access facilities.                                                           #
#                                                                              #
# openvpn.net                                                                  #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing OpenVPN."
echo ""

# Install OpenVPN
apt_function install $OPENVPN

# Configure server.conf.
cat > /etc/openvpn/server.conf << EOF.server.conf
port 1194
proto udp
server 10.8.0.0 $LAN_NETMASK
ifconfig-pool-persist ipp.txt
push "dhcp-option DNS $LAN_IP_ADDRESS"
push "route $LAN_NETWORK_ADDRESS $LAN_NETMASK"
dev tun
crl-verify /etc/ssl/crl.pem
dh /etc/ssl/dh
ca /etc/ssl/ca-cert.pem
cert /etc/ssl/certs/tls-cert.pem
key /etc/ssl/private/tls-key.pem
tls-auth /etc/ssl/private/ta.key 0
keepalive 10 120
user nobody
group nogroup
persist-key
persist-tun
comp-lzo
client-to-client
status openvpn-status.log
verb 3
EOF.server.conf

# Create clientvpn script (run this script to generate client configuration 
# and TLS certificates necessary to connect to your OpenVPN server).
cat > /etc/openvpn/clientvpn << EOF.clientvpn
#!/bin/bash

# Create client.conf.
cat > /etc/openvpn/client.conf << EOF.client.conf
client
dev tun
proto udp
remote $DYNAMICDNS_HOST 1194
resolv-retry infinite
nobind
persist-key
persist-tun
comp-lzo
ca ca-cert.pem
key client-key.pem
cert client-cert.pem
tls-auth ta.key 1
ns-cert-type server
user nobody
group nogroup
verb 3
EOF.client.conf

clear
echo ""
read -p "Type the FQDN of your client (e.g., or-ws1-ub.example.corp), \
followed by [ENTER]: " HOSTNAME

# Create TLS certificate and key.
openssl req -new -nodes -keyout client-key.pem -out newreq.pem \
  -subj "/C=$COUNTRY/ST=$STATE/O=$ORGANIZATION/OU=TLS/CN=\$HOSTNAME/\
emailAddress=$EMAIL_ADDRESS"
openssl ca -batch -out newcert.pem -passin pass:$ADMIN_PASSWORD -infiles newreq.pem
mv newcert.pem client-cert.pem
rm -f newcert.pem newreq.pem

# Archive files to copy over to your client's OpenVPN directory.
cp -u /etc/ssl/ca-cert.pem /etc/ssl/private/ta.key ./
tar -czvf \$HOSTNAME.tar.gz ta.key ca-cert.pem client-cert.pem \
  client-key.pem client.conf
rm -f client.conf ca-cert.pem client-cert.pem client-key.pem ta.key
EOF.clientvpn
chmod 700 /etc/openvpn/clientvpn
chown root:root /etc/openvpn/clientvpn

# Add the nobody user to ssl-cert group so the daemon can read the TLS key.
usermod -aG ssl-cert nobody

# Create shared-secret key.
openvpn --genkey --secret /etc/ssl/private/ta.key
chmod 440 /etc/ssl/private/ta.key
chown nobody /etc/ssl/private/ta.key
chgrp ssl-cert /etc/ssl/private/ta.key

# Reload configuration.
pgrep openvpn > /dev/null
if [ $? -eq 1 ]; then
  systemctl start openvpn
else
  systemctl restart openvpn
fi

################################################################################
#                                                                              #
# The following routine installs and configures Linux Terminal Server Project  #
# (LTSP), which adds thin-client support to Linux servers. LTSP is a flexible, #
# cost effective solution that is empowering schools, businesses, and          #
# organizations all over the world to easily install and deploy desktop        #
# workstations.                                                                #
#                                                                              #
# ltsp.org                                                                     #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing LTSP."
echo ""

# Install LTSP.
apt_function install $LTSP

# Install ltps.conf.
install -m 660 -g sudo /usr/share/ltsp/common/ltsp/ltsp.conf /etc/ltsp/ltsp.conf

# Configure ltsp.conf.
if [ ! -f /etc/ltsp/ltsp.conf.orig ]; then 
  cp /etc/ltsp/ltsp.conf /etc/ltsp/ltsp.conf.orig
fi
sed -i "s|DNS_SERVER=.*$|DNS_SERVER=\"$PDC_LAN_IP_ADDRESS $BDC_LAN_IP_ADDRESS\"|
  s|SEARCH_DOMAIN=.*$|SEARCH_DOMAIN=\"$LAN_DOMAIN\"|" /etc/ltsp/ltsp.conf

# Install iPXE binaries and configuration.
ltsp ipxe

# Configure NFS exports.
ltsp nfs

# Create the ltsp.img initrd add-on.
cp /usr/bin/sshfs /etc/ltsp
ltsp initrd

### How to create a squashfs image from a virtual machine (VM). ###
# After running LinuxSOHO, install a hypervisor such as VirtualBox. Create a VM 
# for your LTSP thin clients, then run the following commands (make sure you 
# change the following path to point to your VM):
# ln -fs ~/VirtualBox\ VMs/ubuntu/ubuntu-flat.vmdk /srv/ltsp/ubuntu.img
# ltsp image ubuntu

################################################################################
#                                                                              #
# The following routine installs and configures Bind DNS for your LAN and WAN. #
# BIND (Berkeley Internet Name Domain) is an implementation of the Domain Name #
# System (DNS) protocols and provides an openly redistributable reference      #
# implementation of the major components of the Domain Name System.            #
#                                                                              #
# isc.org/bind                                                                 #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing Bind DNS."
echo ""

# Install Bind DNS.
apt_function install $BIND

# Configure named.conf.
cat > /etc/bind/named.conf << EOF.named.conf
include "/etc/bind/zones.rfc1918";
include "/etc/bind/named.conf.log";
include "/etc/bind/named.conf.local";
include "/etc/bind/named.conf.options";
include "/etc/bind/named.conf.default-zones";
EOF.named.conf

# Configure DNS forwarders.
NAMESERVERS=$(awk '/^nameserver/{print $2}' /etc/resolv.conf)
NS1=$(echo $NAMESERVERS | awk '{print $1}')
NS2=$(echo $NAMESERVERS | awk '{print $2}')
if [ -z $NS2 ]; then
  NAMESERVERS="$NS1"
else
  NAMESERVERS="$NS1; $NS2"
fi

# Configure named.conf.options.
cat > /etc/bind/named.conf.options << EOF.named.conf.options
options {
  directory "/var/cache/bind";
  forwarders { $NAMESERVERS; 208.67.222.222; 208.67.220.220; 208.67.222.220; \
208.67.220.222; 8.8.8.8; 8.8.4.4; };
  auth-nxdomain no;    # Conform to RFC1035.
  transfer-format many-answers;
  max-transfer-time-in 60;
  notify no;
  version "Not currently available.";
};
EOF.named.conf.options

# Configure named.conf.local.
cat > /etc/bind/named.conf.local << EOF.named.conf.local
include "/etc/bind/rndc.key";

controls {
  inet 127.0.0.1 allow { localhost; } keys { "rndc-key"; };
};

zone "$LAN_DOMAIN" {
  type master;
  file "/var/lib/bind/db.$LAN_DOMAIN.";
  allow-update { key "rndc-key"; };
};

zone "$LAN_REVERSE_ZONE" {
  type master;
  file "/var/lib/bind/db.$LAN_REVERSE_ZONE.";
  allow-update { key "rndc-key"; };
};
EOF.named.conf.local

# Create named.conf.log.
cat > /etc/bind/named.conf.log << EOF.named.conf.log
logging {
  channel update_debug {
    file "/var/log/named/update_debug.log" versions 3 size 100k;
      severity debug;
      print-severity  yes;
      print-time      yes;
    };
    channel security_info {
      file "/var/log/named/security_info.log" versions 1 size 100k;
      severity info;
      print-severity  yes;
      print-time      yes;
    };
    channel bind_log {
    file "/var/log/named/bind.log" versions 3 size 1m;
      severity info;
      print-category  yes;
      print-severity  yes;
      print-time      yes;
    };

    category default { bind_log; };
    category lame-servers { null; };
    category update { update_debug; };
    category update-security { update_debug; };
    category security { security_info; };
};
EOF.named.conf.log
mkdir -p /var/log/named
chown bind /var/log/named

# Create forward DNS zone.
cat << EOF.db.LAN_DOMAIN | column -t > /var/cache/bind/db.$LAN_DOMAIN.
\$TTL 3600  ; 1 hour
\$ORIGIN $LAN_DOMAIN.
@	IN	SOA	$FQDN. hostmaster.$LAN_DOMAIN. (
			$(date +%Y%m%d00) ; serial
			900        ; refresh (15 minutes)
			900        ; retry (15 minutes)
			604800     ; expire (1 week)
			3600       ; minimum (1 hour)
			)
	IN	NS		$FQDN.
$HOSTNAME	IN	A		$LAN_IP_ADDRESS
EOF.db.LAN_DOMAIN

# Create reverse DNS zone.
cat << EOF.db.LAN_REVERSE_ZONE | column -t > /var/cache/bind/db.$LAN_REVERSE_ZONE.
\$TTL 3600  ; 1 hour
\$ORIGIN $LAN_REVERSE_ZONE.
@	IN	SOA	$FQDN. hostmaster.$LAN_DOMAIN. (
			$(date +%Y%m%d00) ; serial
			900        ; refresh (15 minutes)
			900        ; retry (15 minutes)
			604800     ; expire (1 week)
			3600       ; minimum (1 hour)
			)
	IN	NS		$FQDN.
$LAN_IP_ADDRESS_4TH_OCTET	IN	PTR		$FQDN.
EOF.db.LAN_REVERSE_ZONE

# Reload configuration.
systemctl restart bind9

################################################################################
#                                                                              #
# The following routine installs and configures ISC DHCP, a collection of      #
# software that implements all aspects of the DHCP (Dynamic Host Configuration #
# Protocol) suite.                                                             #
#                                                                              #
# isc.org/dhcp                                                                 #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing DHCP Server."
echo ""

# Install ISC DHCP Server.
apt_function install $DHCPD

# Copy /etc/bind/rndc.key to /etc/dhcp/ddns-keys for Apparmor access.
cp -u /etc/bind/rndc.key /etc/dhcp/ddns-keys

# Configure dhcpd.conf.
if [ ! -f /etc/dhcp/dhcpd.conf.orig ]; then 
  mv /etc/dhcp/dhcpd.conf /etc/dhcp/dhcpd.conf.orig
fi
cat > /etc/dhcp/dhcpd.conf << EOF.dhcpd.conf
authoritative;
default-lease-time   600;
max-lease-time       7200;
ddns-updates         on;
ddns-update-style    interim;
include              "/etc/dhcp/ddns-keys/rndc.key";
ignore               client-updates;
update-static-leases on;
use-host-decl-names  on;

zone localhost. {
  primary 127.0.0.1;
  key "rndc-key";
}

zone 0.0.127.in-addr.arpa. {
  primary 127.0.0.1;
  key "rndc-key";
}

# $LAN_NETWORK_ADDRESS/$CIDR
subnet $LAN_NETWORK_ADDRESS netmask $LAN_NETMASK {
  ddns-domainname       "$LAN_DOMAIN.";
  ddns-rev-domainname   "$LAN_REVERSE_ZONE.";
  range                 $DHCP_HOST_MIN $DHCP_HOST_MAX;
  option                broadcast-address $LAN_BROADCAST;
  option                routers $LAN_IP_ADDRESS;
  option                domain-name-servers $LAN_IP_ADDRESS;
  option                domain-name "$LAN_DOMAIN";
  option                ntp-servers $LAN_IP_ADDRESS;
  server-name           $FQDN;
  next-server           $LAN_IP_ADDRESS;
  filename              "ltsp/ltsp.img";
  
  zone $LAN_DOMAIN. {
    primary $LAN_IP_ADDRESS;
    key "rndc-key";
  }
  
  zone $LAN_REVERSE_ZONE. {
    primary $LAN_IP_ADDRESS;
    key "rndc-key";
  }	  
}

host $HOSTNAME {
  hardware ethernet $(ip addr show $LAN_INTERFACE | grep link/ether \
    | awk '{ print $2 }');
  fixed-address $FQDN;
}
EOF.dhcpd.conf

# Set LAN interface.
sed -i "/INTERFACESv4/ c\INTERFACESv4=\"$LAN_INTERFACE\"" \
  /etc/default/isc-dhcp-server

# Reload configuration.
pgrep dhcpd > /dev/null
if [ $? -eq 1 ]; then
  systemctl start isc-dhcp-server
else
  systemctl restart isc-dhcp-server
fi

################################################################################
#                                                                              #
# The following routine installs and configures Snort, Snorby, and Barnyard,   #
# which provide an open-source network intrusion detection and prevention      #
# system capable of performing real-time traffic analysis and packet logging   #
# on IP networks.                                                              #
#                                                                              #
# snort.org                                                                    #
# snorby.org                                                                   #
# github.com/firnsy/barnyard2                                                  #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing Snort, Snorby, and Barnyard."
echo ""

# Install Snort.
apt_function install $SNORT

# Configure Snort's MySQL database.
mysql --user=root --password=$ADMIN_PASSWORD << EOF.snortdb
DROP DATABASE IF EXISTS snortdb;
CREATE DATABASE snortdb;
GRANT CREATE,INSERT,SELECT,DELETE,UPDATE ON snortdb.* TO 'snort'@'localhost' \
  IDENTIFIED BY '$ADMIN_PASSWORD';
FLUSH PRIVILEGES;
EOF.snortdb

# Install Snorby.
apt_function install $SNORBY_DEPENDS
cd /var/www/html
if [ -d snorby ]; then rm -rf snorby; fi
git_function $SNORBY
cd snorby
gem install -V rails
while [ $? -ne 0 ]; do gem install -V rails; done
gem install -V bundler
while [ $? -ne 0 ]; do gem install -V bundler; done
gem update
while [ $? -ne 0 ]; do gem update; done
bundle install --path vendor/bundle
while [ $? -ne 0 ]; do bundle install --path vendor/bundle; done

# Configure Snorby's database.yml.
cp -f config/database.yml.example config/database.yml
sed -i "s|Enter Password Here|$ADMIN_PASSWORD|
  /database:/ c\  database: snorbydb" config/database.yml
chmod 640 config/database.yml

# Configure Snorby's snorby_config.yml.
cp -u config/snorby_config.yml.example config/snorby_config.yml
sed -i "s|baseuri: ''|baseuri: '/snorby'|
  s|demo.snorby.org|$LAN_IP_ADDRESS|
  s|/usr/local/bin/wkhtmltopdf|/usr/bin/wkhtmltopdf|
  s|ssl: false|ssl: true|
  s|snorby@snorby.org|root@localhost|
  s|- \"\"|- \"/etc/snort/rules\"|" config/snorby_config.yml

# Configure Snorby's seeds.rb.
sed -i "/User.create(:name =>/ c\User.create(:name => 'Administrator', \
:email => '$EMAIL_ADDRESS', :password => '$ADMIN_PASSWORD', \
:password_confirmation => '$ADMIN_PASSWORD', :admin => true) if User.all.blank?
  /Setting.set(:company/ c\Setting.set(:company, '$ORGANIZATION') \
unless Setting.company?
  /Setting.set(:email/ c\Setting.set(:email, '$EMAIL_ADDRESS') \
unless Setting.email?" db/seeds.rb
chmod 640 db/seeds.rb

# Configure Snorby's mail.config.rb.
cat > config/initializers/mail_config.rb << EOF.mail.config.rb
ActionMailer::Base.delivery_method = :sendmail
ActionMailer::Base.sendmail_settings = {
  :location => '/usr/sbin/sendmail',
  :arguments => '-i -t'
}

ActionMailer::Base.perform_deliveries = true
ActionMailer::Base.raise_delivery_errors = true
EOF.mail.config.rb

# Setup the database and the Snorby defaults.
bundle exec rake snorby:setup RAILS_ENV=production

# Set permissions.
chown -R www-data:www-data /var/www/html/snorby

# Add Phusion Passenger configuration for Snorby.
cat > /etc/apache2/conf-available/snorby.conf << EOF.snorby.conf
Alias /snorby /var/www/html/snorby/public
<Location /snorby>
  PassengerBaseURI /snorby
  PassengerAppRoot /var/www/html/snorby
</Location>

<Directory /var/www/html/snorby/public>
  Options -MultiViews
  Require ip 127.0.0.0/8 $LAN_NETWORK_ADDRESS/$CIDR
  SSLRequireSSL
</Directory>
EOF.snorby.conf
a2enconf snorby

# Install Barnyard.
apt_function install $BARNYARD_DEPENDS
cd /usr/local/src
if [ -d barnyard2 ]; then rm -rf barnyard2; fi
git_function $BARNYARD
cd barnyard2
ln -fs /usr/include/dumbnet.h /usr/include/dnet.h
./autogen.sh
./configure --with-mysql \
  --with-mysql-libraries=/usr/lib/$HARDWARE_PLATFORM-linux-gnu
make
make install
mysql --user=root --password=$ADMIN_PASSWORD -D snortdb < ./schemas/create_mysql
cp -u etc/barnyard2.conf /etc/snort
cd .. && rm -rf barnyard2

# Configure baryard2.conf.
if [ ! -f /etc/snort/barnyard2.conf.orig ]; then 
  cp /etc/snort/barnyard2.conf /etc/snort/barnyard2.conf.orig
fi
sed -i "s|#config hostname:   thor|config hostname:   localhost|
  s|#config interface:  eth0|config interface:  $LAN_INTERFACE|
  /output database: log, mysql/ c\output database: log, mysql, user=snort \
password=$ADMIN_PASSWORD dbname=snortdb host=localhost" \
  /etc/snort/barnyard2.conf
chmod 640 /etc/snort/barnyard2.conf
chown root:snort /etc/snort/barnyard2.conf

# Configure snort.conf.
if [ ! -f /etc/snort/snort.conf.orig ]; then 
  cp -u /etc/snort/snort.conf /etc/snort/snort.conf.orig
fi
sed -i "/ipvar EXTERNAL_NET any/ s|^|#|
  /ipvar EXTERNAL_NET !\$HOME_NET/ s|^#||" /etc/snort/snort.conf

# Configure snort.debian.conf.
if [ ! -f /etc/snort/snort.debian.conf.orig ]; then 
  cp /etc/snort/snort.debian.conf /etc/snort/snort.debian.conf.orig
fi
sed -i "s|DEBIAN_SNORT_HOME_NET=.*$\
|DEBIAN_SNORT_HOME_NET=\"$LAN_NETWORK_ADDRESS/$CIDR\"|
  s|DEBIAN_SNORT_INTERFACE=.*$|DEBIAN_SNORT_INTERFACE=\
\"$LAN_INTERFACE $WAN_INTERFACE\"|" /etc/snort/snort.debian.conf

# Remove the pending Snort database configuration file.
rm -f /etc/snort/db-pending-config

# Configure oinkmaster.conf.
sed -i "/Community-Rules-CURRENT.tar.gz/ s|^# ||" /etc/oinkmaster.conf

# Create Oinkmaster cron job.
echo "@daily root oinkmaster -o /etc/snort/rules \
-b /etc/snort/backup 2>&1 | logger -t oinkmaster" > \
  /etc/cron.d/oinkmaster_updater

# Create cron job to automatically update Snorby.
cat > /usr/local/bin/snorby_updater.sh << EOF.snorby_updater.sh
cd /var/www/html/snorby
git pull origin master
rake snorby:update
gem update
exit 0
EOF.snorby_updater.sh
chmod 700 /usr/local/bin/snorby_updater.sh
echo "@daily root /usr/local/bin/snorby_updater.sh" > /etc/cron.d/snorby_updater

# Reload configuration.
systemctl restart apache2
systemctl restart snort

################################################################################
#                                                                              #
# The following routine installs and configures Postfix, a scalable, secure    #
# implementation of an SMTP mail transfer agent.                               #
#                                                                              #
# postfix.org                                                                  #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing Postfix."
echo ""

# Install Postfix.
apt_function install $POSTFIX

# Create root alias, and update the local alias database.
if [ ! -f /etc/aliases.orig ]; then 
  cp /etc/aliases /etc/aliases.orig
fi
grep -q "root: $EMAIL_ADDRESS" /etc/aliases || \
  echo "root: $EMAIL_ADDRESS" >> /etc/aliases
newaliases

# Configure mailname.
echo "$FQDN" > /etc/mailname

# Configure main.cf.
if [ ! -f /etc/postfix/main.cf.orig ]; then 
  cp /etc/postfix/main.cf /etc/postfix/main.cf.orig
fi
cat > /etc/postfix/main.cf << EOF.main.cf
myhostname = $FQDN
alias_maps = hash:/etc/aliases
alias_database = hash:/etc/aliases
myorigin = /etc/mailname
mydestination = \$myhostname, localhost.\$mydomain, localhost
inet_interfaces = $LAN_IP_ADDRESS
mynetworks = 127.0.0.0/8 $LAN_NETWORK_ADDRESS/$CIDR
mailbox_command = procmail -a "\$EXTENSION"
mailbox_size_limit = 0
recipient_delimiter = +
EOF.main.cf

# Reload configuration.
systemctl restart postfix

################################################################################
#                                                                              #
# The following routine installs and configures Asterisk and Asterisk-GUI.     #
#                                                                              #
# Asterisk provides an open-source communications project that converts an     #
# ordinary computer into a feature-rich voice communications server. Asterisk  #
# makes it simple to create and deploy a wide range of telephony applications  #
# and services, including IP PBXs, VoIP gateways, call center ACDs, and IVR    #
# systems.                                                                     #
#                                                                              #
# Asterisk-GUI provides a graphical user interface to manage Asterisk.         #
#                                                                              #
# asterisk.org                                                                 #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing Asterisk."
echo ""

# Install Asterisk.
apt_function install $ASTERISK_DEPENDS $ASTERISK

# Install PyGoogleVoice.
apt_function install $PYGOOGLEVOICE_DEPENDS
easy_install simplejson
while [ $? -ne 0 ]; do easy_install simplejson; done
easy_install -U pygooglevoice
while [ $? -ne 0 ]; do easy_install -U pygooglevoice; done

# Configure rtp.conf.
if [ ! -f /etc/asterisk/rtp.conf.orig ]; then 
  cp /etc/asterisk/rtp.conf /etc/asterisk/rtp.conf.orig
fi
sed -i "s|rtpstart=10000|rtpstart=30000|
  s|rtpend=20000|rtpend=40000|" /etc/asterisk/rtp.conf

# Create http.conf.
if [ ! -f /etc/asterisk/http.conf.orig ]; then 
  cp /etc/asterisk/http.conf /etc/asterisk/http.conf.orig
fi
cat > /etc/asterisk/http.conf << EOF.http.conf
[general]
enabled = yes
bindaddr = $LAN_IP_ADDRESS
enablestatic = yes
tlsenable = yes
tlsbindaddr=$LAN_IP_ADDRESS:8089
tlscertfile = /etc/ssl/certs/tls-cert.pem
tlsprivatekey = /etc/ssl/private/tls-key.pem

[post_mappings]
backups = /var/lib/asterisk/gui_backups
moh = /var/lib/asterisk/moh
EOF.http.conf
chown asterisk:asterisk /etc/asterisk/http.conf

# Create manager.conf.
if [ ! -f /etc/asterisk/manager.conf.orig ]; then 
  cp /etc/asterisk/manager.conf /etc/asterisk/manager.conf.orig
fi
cat > /etc/asterisk/manager.conf << EOF.manager.conf
[general]
enabled = yes
webenabled = yes
port = 5038
bindaddr = $LAN_IP_ADDRESS

[admin]
secret = $ADMIN_PASSWORD
read = system,call,log,verbose,command,agent,config
write = system,call,log,verbose,command,agent,config,originate
EOF.manager.conf
chown root:asterisk /etc/asterisk/manager.conf
chmod 640 /etc/asterisk/manager.conf

# Install Asterisk-GUI.
cd /usr/local/src
svn_function $ASTERISK_GUI asterisk-gui
cd asterisk-gui
sh configure && make && make install && make checkconfig
cd ..
rm -rf asterisk-gui /usr/share/asterisk/static-http
ln -fs /var/lib/asterisk/static-http /usr/share/asterisk/static-http
chown -R asterisk:asterisk /var/lib/asterisk

# Add the Asterisk user to ssl-cert group so the daemon can read the TLS key.
usermod -aG ssl-cert asterisk

# Reload configuration.
sed -i "/RUNASTERISK=yes/ s|^#||" /etc/default/asterisk
systemctl restart asterisk

################################################################################
#                                                                              #
# The following routine installs and configures Webmin, a web-based interface  #
# for system administration. Using any browser that supports tables and forms  #
# (and Java for the File Manager module), you can setup user accounts, Apache, #
# DNS, file sharing and so on.                                                 #
#                                                                              #
# webmin.com                                                                   #
#                                                                              #
################################################################################

clear
echo ""
echo "Installing Webmin."
echo ""

# Install Webmin.
apt_function install $WEBMIN

# Configure miniserv.conf.
if [ ! -f /etc/webmin/miniserv.conf.orig ]; then 
  cp /etc/webmin/miniserv.conf /etc/webmin/miniserv.conf.orig
fi
sed -i "/keyfile=/ c\keyfile=/etc/ssl/private/tls-key.pem" \
  /etc/webmin/miniserv.conf
grep -q "certfile" /etc/webmin/miniserv.conf || \
  sed -i "/keyfile=/ \
a\certfile=/etc/ssl/certs/tls-cert.pem\nssl_redirect=1" \
    /etc/webmin/miniserv.conf
grep -q "allow=127.0.0.1 LOCAL" /etc/webmin/miniserv.conf || \
  echo "allow=127.0.0.1 LOCAL" >> /etc/webmin/miniserv.conf

# Configure miniserv.users.
echo "admin:x:0::::::::" > /etc/webmin/miniserv.users

# Configure DHCP.
if [ ! -f /etc/webmin/dhcpd/config.orig ]; then 
  cp /etc/webmin/dhcpd/config /etc/webmin/dhcpd/config.orig
fi
sed -i "/pid_file=/ c\pid_file=/run/dhcp-server/dhcpd.pid" \
  /etc/webmin/dhcpd/config

# Configure Grub.
if [ ! -f /etc/webmin/grub/config.orig ]; then 
  cp /etc/webmin/grub/config /etc/webmin/grub/config.orig
fi
sed -i "/menu_file=/ c\menu_file=/etc/grub.d/40_custom
  /grub_path=/ c\grub_path=/usr/sbin/grub-mkconfig" /etc/webmin/grub/config

# Configure Stunnel.
if [ ! -f /etc/webmin/config.orig ]; then 
  cp /etc/webmin/config /etc/webmin/config.orig
fi
sed -i "/stunnel_path=/ c\stunnel_path=/usr/bin/stunnel4
  /pem_path=/ c\pem_path=/etc/ssl/certs/tls-cert.pem" /etc/webmin/config
sed -i "s|ENABLED=0|ENABLED=1|" /etc/default/stunnel4
cp -u /usr/share/doc/stunnel4/examples/stunnel.conf-sample \
  /etc/stunnel/stunnel.conf
if [ ! -f /etc/stunnel/stunnel.conf.orig ]; then 
  cp /etc/stunnel/stunnel.conf /etc/stunnel/stunnel.conf.orig
fi
sed -i "/cert =/ c\cert = /etc/ssl/certs/tls-cert.pem
  /key =/ c\key = /etc/ssl/private/tls-key.pem
  /CAfile =/ c\CAfile = /etc/ssl/ca-cert.pem
  /CRLfile =/ c\CRLfile = /etc/ssl/crl.pem" /etc/stunnel/stunnel.conf

# Set administrator password for MySQL.
if [ ! -f /etc/webmin/mysql/config.orig ]; then 
  cp /etc/webmin/mysql/config /etc/webmin/mysql/config.orig
fi
grep -q "pass=$ADMIN_PASSWORD" /etc/webmin/mysql/config || \
  echo "pass=$ADMIN_PASSWORD" >> /etc/webmin/mysql/config
chmod 600 /etc/webmin/mysql/config

# Configure admin.acl.
cat > /etc/webmin/admin.acl << EOF.admin.acl
rpc=2
nodot=0
webminsearch=1
uedit_mode=0
gedit_mode=0
feedback=2
otherdirs=
readonly=0
fileunix=root
uedit=
negative=0
root=/
uedit2=
gedit=
gedit2=
EOF.admin.acl

echo "admin: backup-config change-user webmincron usermin webminlog webmin \
servers acl init passwd quota mount fsdump logrotate mailcap \
pam proc at cron package-updates software man syslog system-status useradmin \
security-updates virtualmin-awstats apache bind8 dhcpd mysql postfix procmail \
mailboxes sshd samba squid sarg virtual-server webalizer bandwidth exports \
net xinetd inetd stunnel shorewall tcpwrappers idmapd filter burner grub lilo \
raid lvm fdisk lpadmin smart-status time vgetty cluster-passwd cluster-copy \
cluster-cron cluster-shell cluster-software cluster-usermin cluster-useradmin \
cluster-webmin shell custom file tunnel phpini cpan htaccess-htpasswd telnet \
status ajaxterm updown proftpd" > /etc/webmin/webmin.acl

# Configure installed.cache.
cat > /etc/webmin/installed.cache << EOF.installed.cache
quota=1
proc=1
tcpwrappers=1
ajaxterm=1
mailboxes=1
usermin=1
bind8=1
stunnel=1
nis=1
at=1
raid=1
lpadmin=1
man=1
backup-config=1
postfix=1
sshd=1
telnet=1
software=1
lvm=1
custom=1
cpan=1
shell=1
grub=1
shorewall=1
file=1
mailcap=1
phpini=1
init=1
webminlog=1
webmin=1
idmapd=1
apache=1
fdisk=1
syslog=1
procmail=1
smart-status=1
webalizer=1
pam=1
updown=1
samba=1
filter=1
xinetd=1
webmincron=1
mount=1
acl=1
inetd=1
dhcpd=1
system-status=1
passwd=1
htaccess-htpasswd=1
change-user=1
status=1
mysql=1
logrotate=1
bandwidth=1
burner=1
time=1
package-updates=1
cron=1
useradmin=1
squid=1
exports=1
net=1
firewall=1
servers=1
tunnel=1
fsdump=1
sarg=1
proftpd=1
EOF.installed.cache

# Reload configuration.
systemctl restart webmin
systemctl restart stunnel4
systemctl restart apache2

# Create web-access menu.
cat > /var/www/html/index.html << EOF.index.html
<html>
  <body>
    <div align="center">
      <a href="https://$LAN_IP_ADDRESS:8089/static/config/index.html" \
target="_blank">Asterisk</a> |
      <a href="https://$LAN_IP_ADDRESS:631" target="_blank">CUPS</a> |
      <a href="https://$LAN_IP_ADDRESS/munin" target="_blank">Munin</a> |
      <a href="https://$LAN_IP_ADDRESS/nagios4" target="_blank">Nagios</a> |
      <a href="https://$LAN_IP_ADDRESS/phpmyadmin" target="_blank">phpMyAdmin</a> |
      <a href="https://$LAN_IP_ADDRESS/snorby" target="_blank">Snorby</a> |
      <a href="https://$LAN_IP_ADDRESS/squidguardmgr" \
target="_blank">SquidGuardMgr</a> |
      <a href="https://$LAN_IP_ADDRESS:10000" target="_blank">Webmin</a>
    </div>
  </body>
</html>
EOF.index.html

echo "Script end time: $(date +%c)"

unset DEBIAN_FRONTEND

# Login to the GUI.
clear
echo ""
echo "Access your Ubuntu Linux server at https://$LAN_IP_ADDRESS"
echo ""
echo "Login to the GUI with the following:"
echo ""
echo "username = admin"
echo "password = enter your administrator password"
echo ""
echo "Snorby username = $EMAIL_ADDRESS"
echo ""

# End of redirect (STDOUT and STDERROR logged to terminal and install.log).
) 2>&1 | tee install.log 

exit 0
